#!/usr/bin/python
#
# This file is a portion of the Red Hat Update Agent
# Copyright (c) 1999, 2000 Red Hat, Inc.  Distributed under GPL
#
# $Id: config.py,v 1.33.2.2 2002/09/27 03:34:56 alikins Exp $
"""This module includes the Config and Up2date Config classes use
by the up2date agent to hold config info."""

import string, os


class ConfigChain:
    def __init__(self):
        # list of config sources, the highest being
        # first in the list
        self.source_list = []

    # add a Config() derived source class to the list
    # add them to the top of the list by default. #
    # ie, add stores in order like default, configfile, other, runtime
    def addConfigSource(self, source, name, index=0):
        self.source_list.insert(index,(source, name))

    # add support for reading from a particular source?
    def readEntry(self, name, sourceName=None):
        if sourceName:
            source, source_info = self.__findSource(sourceName)
            if source.has_key(name):
                return source.readEntry(name)
        else:    
            for source, source_info in self.source_list:
                if source.has_key(name):
                    return source.readEntry(name)
                
        return None

    def setEntry(self, name, value):
        # setEntry always sets the value in the temporary
        # config store, which should be the first one
        # in the list
        source , source_info = self.source_list[0]
        source.setEntry(name, value)

    def __findSource(self, sourceName):
        sourceNames = map(lambda a:a[1], self.source_list)
        index = None
        index = sourceNames.index(sourceName)
        return self.source_list[index]

    def saveEntry(self, name, value, sourceName=None):
        # if we specify a specify source...
        if sourceName:
            source, source_info  = self.__findSource(sourceName)
            # this should throw some sort of exception if it cant save
            source.saveEntry(name, value)
        else:
            # assume we are writing out to the last entry in the list
            # which will be the config file by default
            # or is it the defaults the last?
            defaultName = "configFile"
            source, source_info = self.__findSource(defaultName)
            source.saveEntry(name, value)

    # FIXME
    def writeEntry(self, name, value):
        # write to the configfile always for now...
        # TODO/FIXME: make this an option to do a xmlrpc call to
        # the server to store the config
        defaultName = "configFile"
        source, source_info = self.__findSource(defaultName)
	source.setEntry(name, value)

    def save(self, sourceName="configFile"):
        source, source_info = self.__findSource(sourceName)
        source.save()

    def keys(self, sourceName="configFile"):
        source, source_info = self.__findSource(sourceName)
        return source.keys()

    def values(self, sourceName="configFile"):
        source, source_info = self.__findSource(sourceName)
        return source.values()

    def readValues(self, sourceName="configFile"):
	source, source_info = self.__findSource(sourceName)
        return source.readValues()

    # use instead of __getitem__ since it's useage is differnt (sourcename)
    def getItem(self, key, sourceName="configFile"):
        source, source_info = self.__findSource(sourceName)
        return source.dict[key]

    def setItem(self, key, value, sourceName="configFile"):
        source, source_info = self.__findSource(sourceName)
        source.dict[key] = value
        
    def hasEntry(self, name, sourceName=None):
        # if we specify a specify source...
        if sourceName:
            source, source_info  = self.__findSource(sourceName)
            if source.has_key(name):
                return 1
            else:
                return 0
        else:
            for source, source_info in self.source_list:
                if source.has_key(name):
                    return 1

        # nothing had that key
        return 0


up2dateConfigInstance = None
#print "sdffffffff"
#print up2dateConfigInstance
class Config:
    "class for handling global config options for the client"
    def __init__(self):
	self.dict = {}
	self.fileName = None

    def reload(self):
        self.load()
        
    def load(self):
	if self.fileName == None:
	    return

        if not os.access(self.fileName, os.R_OK):
            print "warning: can't access %s" % self.fileName
            return
        
        f = open(self.fileName, "r")

	key = ""
	comment = ""
	value = ""
	for line in f.readlines():
            line = string.strip(line)
            if line == '':
		continue
	    if line[0] == "#":
		continue
	    try:
		(key, value) = map(string.strip, string.split(line, "=", 1))
	    except ValueError:
                # Bad directive: not in 'a = b' format
		continue

	    cf = 0
	    pos = string.find(key, "[comment]")
	    if pos != -1:
		key = key[:pos]
		cf = 1

	    if cf:
		comment = value
	    else:
		# possibly split value into a list
		values = string.split(value, ";")
		if len(values) == 1:
		    try:
			value = int(value)
		    except ValueError:
			pass
		    self.dict[key] = (comment, value)
		elif values[0] == "":
		    self.dict[key] = (comment, [])
		else:
		    self.dict[key] = (comment, values[:-1])
		comment = ""

	f.close()


    def save(self):
	if self.fileName == None:
	    return
        
        f = open(self.fileName, "w")
        os.chmod(self.fileName, 0600)

	f.write("# Automatically generated Red Hat Update Agent config file, do not edit.\n")
	f.write("# Format: 1.0\n")
	f.write("")
	for key in self.dict.keys():
	    val = self.dict[key]
	    f.write("%s[comment]=%s\n" % (key, val[0]))
	    if type(val[1]) == type([]):
		f.write("%s=%s;\n" % (key, string.join(map(str, val[1]), ';')))
	    else:
		f.write("%s=%s\n" % (key, val[1]))
	    f.write("\n")

	f.close()
	    
    def has_key(self, name):
        return self.dict.has_key(name)

    def keys(self):
	return self.dict.keys()

    def values(self):
	return self.dict.values()

    # return a dict of just param:value
    def readValues(self):
	tmpdict = {}
	for key in self.dict.keys():
	    tmpdict[key] = self.dict[key][1]
	return tmpdict

    def readEntry(self, name):
	try:
	    val = self.dict[name]
	except:
	    val = (None, None)

	return val[1]

    def saveEntry(self, name, value):
        # set and save permanantely
        self.setEntry(name, value)
        self.save()

    def writeEntry(self, name, value):
	self.setEntry(name, value)

    # temporarily set the value
    def setEntry(self, name, value):
        
	try:
	    val = self.dict[name]
	except:
	    val = (None, None)

	val = (val[0], value)
	self.dict[name] = val

    def hasEntry(self, name):
        return self.dict.has_key(name)

class RegisterPeekConfig(Config):
    "class to peek into the rhn_register configuration to see if we should use any of it's values"
    def __init__(self):
        Config.__init__(self)
        self.fileName = "/etc/sysconfig/rhn/rhn_register"
        
    def save(self):
        "just to be on the safe side..."
        pass

class Up2dateConfig(Config):
    def __init__(self):
	Config.__init__(self)

        self.fileName = "/etc/sysconfig/rhn/up2date"
        self.dict['enableProxy'] = ("Use a HTTP Proxy", 0)
        self.dict['serverURL'] = ("Remote server URL",
                                  "https://xmlrpc.rhn.redhat.com/XMLRPC")
        self.dict['noSSLServerURL'] = ("Remote server URL without SSL",
                                       "http://xmlrpc.rhn.redhat.com/XMLRPC")

	self.dict['debug'] = ("Whether or not debugging is enabled", 0)
        self.dict['systemIdPath'] = ("Location of system id",
                                     "/etc/sysconfig/rhn/systemid")
	self.dict['adminAddress'] = ("List of e-mail addresses for update agent to communicate with when run in batch mode", ["root@localhost"])
	self.dict['storageDir'] = ("Where to store packages and other data when they are retrieved", "/var/spool/up2date")
	self.dict['pkgSkipList'] = ("A list of package names, optionally including wildcards, to skip", ["kernel*"])
	self.dict['removeSkipList'] = ("A list of package names, optionally including wildcards, that up2date will not remove", ["kernel*"])
	self.dict['fileSkipList'] = ("A list of file names, optionally including wildcards, to skip", [])
	self.dict['noReplaceConfig'] = ("When selected, no packages that would change configuration data are automatically installed", 1)
	self.dict['retrieveOnly'] = ("Retrieve packages only", 0)
        self.dict['retrieveSource'] = ("Retrieve source RPM along with binary package", 0)
	self.dict['keepAfterInstall'] = ("Keep packages on disk after installation", 0)
	self.dict['versionOverride'] = ("Override the automatically determined system version", "")
        self.dict['useGPG'] = ("Use GPG to verify package integrity", 1)
	self.dict['headerCacheSize'] = ("The maximum number of rpm headers to cache in ram", 40)
	self.dict['headerFetchCount'] = ("The maximimum number of rpm headers to fetch at once", 10)
	self.dict['forceInstall'] = ("Force package installation, ignoring package, file and config file skip list", 0)
        self.dict['httpProxy'] = ("HTTP proxy in host:port format, e.g. squid.redhat.com:3128", "")
        self.dict['proxyUser'] = ("The username for an authenticated proxy", "")
        self.dict['proxyPassword'] = ("The password to use for an authenticated proxy", "")
        self.dict['enableProxyAuth'] = ("To use an authenticated proxy or not", 0)
        self.dict['noBootLoader'] = ("To disable modification of the boot loader (lilo, silo, etc)", 0)	
        self.dict['networkRetries'] = ("Number of attempts to make at network connections before giving up", 5)
        self.dict['sslCACert'] = ("The CA cert used to verify the ssl server", "/usr/share/rhn/RHNS-CA-CERT")
	self.dict['gpgKeyRing'] = ("The location of the gpg keyring to use for package checking.", "/etc/sysconfig/rhn/up2date-keyring.gpg")


# just a temporary, non persistent stash for runtime variables
class Up2dateConfigRuntime(Config):
    def __init__(self):
        Config.__init__(self)


def initUp2dateConfig():
    global up2dateConfigInstance

    if up2dateConfigInstance == None:
        chain = ConfigChain()
        up2dateConfigInstance = Up2dateConfig()
        up2dateConfigRuntime = Up2dateConfigRuntime()
        up2dateConfigInstance.load()
        chain.addConfigSource(up2dateConfigInstance, "configFile")
        chain.addConfigSource(up2dateConfigRuntime, "runtime")
        up2dateConfigInstance = chain

    return up2dateConfigInstance

def main():
    source = initUp2dateConfig()

    print source.readEntry("serverURL")
    source.setEntry("serverURL", "http://hokeypokeyland.com")
    print source.readEntry("serverURL")
    print source.saveEntry("debug", "sdfasdfasdfasdf")
    source.save()
    print source.readEntry("debug")


if __name__ == "__main__":
    main()


