#!/usr/bin/python

import os
import config
import up2dateErrors
import up2dateMessages
import up2dateLog
import rpmUtils
import re


cfg = config.initUp2dateConfig()

log = up2dateLog.initLog()

# directory to look for .gnupg/options, etc
gpg_home_dir = "/root/.gnupg"


def checkGPGInstallation():
    if not os.access("/usr/bin/gpg", os.X_OK):
        log.log_me("error: GPG is not installed")
        # not installed or accessible.
        return 1

    gpg_command = "/usr/bin/gpg %s --list-keys 2>/dev/null" % rpmUtils.getGPGflags() 
    output = os.popen(gpg_command, "r").read()
    # FIXME: add support for checking that all needed keys are installed on the keyrings
    # need a way to configure which keys are acceptable
    if not re.search("pub  1024D/DB42A60E 1999-09-23 Red Hat, Inc <security@redhat.com>", output):
        return 2

    return 0

# key is which key we would like to add to the keyring
def addGPGKey(key="/usr/share/rhn/RPM-GPG-KEY"):
    # might as well log this security stuff well
    log.log_me("Adding %s to keyring" % key)
    gpg_command = "/usr/bin/gpg %s --import %s > /dev/null 2>&1" % (rpmUtils.getGPGflags(), key)
    log.log_me(gpg_command)
    return os.system(gpg_command)

def checkGPGSanity():
    if cfg.readEntry("useGPG") and checkGPGInstallation() == 1:
        errMsg = up2dateMessages.gpgWarning1Msg
        
        print errMsg
        raise up2dateErrors.GPGInstallationError(errMsg)

    if cfg.readEntry("useGPG") and checkGPGInstallation() == 2:
        errMsg = up2dateMessages.gpgWarning2Msg  % rpmUtils.getGPGflags()
        print errMsg
        raise up2dateErrors.GPGKeyringError(errMsg)
