#!/usr/bin/python

# class defination for the HeaderList and maybe Header objects?
#
#  Ideally, this will include it's own memory and disk caching, so
#  that the retrieveal classes can be made more generic
#

import sys
import config
import up2dateLog
import rpcServer
import rpmSource

# get a reference to the log file object
log = up2dateLog.initLog()

#grab an instance of the config class shared
cfg = config.initUp2dateConfig()



# small class for header caching pretending to be a dict
# in theory we could eventually make this smart and stuff
# (ie, mru, etc)
#
# implement it this way for now so we can make a max header
# cache size
#
# Need a miniHeaderCache, which caches all headers
# but only: name,version,release,arch,summary,description,size
# Otherwise the size computations are slow as dirt
#
class HeaderCache:
    def __init__(self):
        # how many headers to cache in ram
        if cfg.readEntry("headerCacheSize"):
            self.cache_size = cfg.readEntry("headerCacheSize")
        else:
            self.cache_size = 30
        self.__cache = {}
        self.__cacheLite = {}

    def set_cache_size(self, number_of_headers):
        self.cache_size = number_of_headers

    def __liteCopy(self, header):
        tmp = {}
        tmp['name'] = header['name']
        tmp['version'] = header['version']
        tmp['release'] = header['release']
        tmp['arch'] = header['arch']
        tmp['summary'] = header['summary']
        tmp['description'] = header['description']
        tmp['size'] = header['size']
        return tmp

    def __setitem__(self,item,value):
        if len(self.__cache) <= self.cache_size:
            self.__cache[item] = value
            self.__cacheLite[item] = self.__liteCopy(value)
        else:
            # okay, this is about as stupid of a cache as you can get
            # but if we hit the max cache size, this is as good as
            # any mechanism for freeing up space in the cache. This
            # would be a good place to put some smarts
            bar = self.__cache.keys()
            del self.__cache[bar[self.cache_size-1]]
            self.__cache[item] = value
            self.__cacheLite[item] = self.__liteCopy(value)

    def __getitem__(self, item):
        return self.__cache[item]
    

    def getLite(self, item):
        return self.__cacheLite[item]

    def __len__(self):
        return len(self.__cache)

    def keys(self):
        return self.__cache.keys()

    def values(self):
        return self.__cache.keys()

    def has_key(self, item,lite=None):
#        print "\n########\nhas_key called\n###########\n"
#        print "item: %s" % item
#        print self.__cache.keys()
        
        if lite:
            return self.__cacheLite.has_key(item)
        else:
            return self.__cache.has_key(item)

    def __delitem__(self, item):
        del self.__cache[item]

    def printLite(self):
        print self.__cacheLite

# we do all the list to tuple munging here. I dont want
# to have to force it at a higher level, and I dont want
# headerCache to have to care
class HeaderList:
    _cache = HeaderCache()
    def __init__(self):
        pass

    def cache(self):
        return id(self._cache)

    # items are in the format of a tuple of format
    # (name, version, release, epoch, arch)
    def __getitem__(self,item):
        itemtuple = tuple(item[:5])
#        print "HeaderList::getitem: item: %s" % item
        if self._cache.has_key(itemtuple):
            return self._cache[itemtuple]
        else:
            self.__retrievePackage(item)
            return self._cache[itemtuple]

    def __retrievePackage(self,item):
        # need to make sure headerSource isnt populating the cache as well
        hdr, sourceType = rpcServer.doCall(rpmSource.psc.headerSource.getHeader, item)
        # always populate the cache when we fetch a package
        self._cache[tuple(item[:5])] = hdr

    def __setitem__(self,item,value):
        self._cache[tuple(item[:5])] = value

    def getLite(self, item):
        return self._cache.getLite(tuple(item[:5]))

    def keys(self):
        return self._cache.keys()

    def values(self):
        return self._cache.values()

    def __delitem__(self, item):
        del self._cache[tuple(item[:5])]
