#!/usr/bin/python
#
# Package list mangling
#
# $Id: packageList.py,v 1.9.4.6 2002/10/04 07:19:01 alikins Exp $

import os
import fnmatch
import string
import sys
import rpm
# temp home for all the package list related functions
# move this to a class
import up2dateUtils
import up2dateLog
import config
import rpmUtils
import headers
import rhnPackageInfo

from translate import _
# get a reference to the log file object
log = up2dateLog.initLog()

#grab an instance of the config class shared
cfg = config.initUp2dateConfig()

headerList = headers.HeaderList()


# idea... a class to wrap everything in here...
#   we add all the info we need, and then call PackageList.getTransaction or
#    PackageList.runTransaction or something
#
#   needs to be able to handle adding packages for install/update/remove
#   and some set of standard options for those (--force?, --nodeps, --justdb,etc)
#
#   At the very least, this needs to be able to do what getUpdatedPackageList does
#
#
class PackageList:
    def __init__(self):
        # rpm transaction set flags
        self.nodeps = None
        self.force = None
        self.justdb = None
        # A hash keyed on the package name, and with a list of all the
        # packages with that name as a value
        self.__instPackagesHash = {}
        # A list of available packages
        self.__availPackagesList = []
        # A hash keyed on the package name, None as value
        self.__forcePackagesHash = {}
        # A hash keyed on the package (n, v, r, e, a), a list of
        # (n, v, r, e, a, obsol name, obsol ver, obsol sense) as values
        self.__obsolPackagesHash = {}
        # list of packages we should update but are marked to
        # be skipped for some reason. for ui stuff
        # (names with possible wildcards)
        self.__skipPatternList = []
        # Skipped packages
        self.__skippedPackagesList = []
        self.__obsoletedPackagesList = []
        self.__installedObsoletingPackageList = []

        self.msgCallback = None
        self.progressCallback = None
        self.refreshCallback = None
        # bit of a kluge to make a better ui
        self.ignoreMsgCallback = 1

	self.debug = 0 

    def run(self):
        # based on getpdatedPackageList
        installedPackageList = rpmUtils.getInstalledPackageList()
        self.addInstalledPackages(installedPackageList)
        availList = rhnPackageInfo.availablePackageList(self.msgCallback, self.progressCallback)

        if availList:
            self.addAvailablePackages(availList)

        obsList = rhnPackageInfo.obsoletesList(msgCallback = self.msgCallback,
                                               progressCallback = self.progressCallback )
        self.addObsoletePackages(obsList)
        if not cfg.readEntry("forceInstall"):
            self.addSkipPatterns(cfg.readEntry("pkgSkipList"))
        
        # skipped packages and the packages to install
        # are self.getSkippedPackages and self.getPackagesToInstall

    def addInstalledPackages(self, installedPackages):
        for p in installedPackages:
            pName = p[0]
            if not self.__instPackagesHash.has_key(pName):
                self.__instPackagesHash[pName] = []
            self.__instPackagesHash[pName].append(p)

    def addAvailablePackages(self, availablePackages):
        for p in availablePackages:
            self.__availPackagesList.append(p)
        #print self.__availPackagesList

    def addForcePackages(self, forcePackages):
        for p in forcePackages:
            self.__forcePackagesHash[p] = None

    def addObsoletePackages(self, obsoletePackages):
        for p in obsoletePackages:
            key = tuple(p[:5])
            if not self.__obsolPackagesHash.has_key(key):
                self.__obsolPackagesHash[key] = []
            self.__obsolPackagesHash[key].append(p)
        
        self.__obsoletedPackageHash = {}
        for key in self.__obsolPackagesHash.keys():
            for obsInfo in self.__obsolPackagesHash[key]:            
                if not self.__obsoletedPackageHash.has_key(obsInfo[5]):
                    self.__obsoletedPackageHash[obsInfo[5]] = []
                    self.__obsoletedPackageHash[obsInfo[5]].append(obsInfo)
            # foo[obsoletingpackage] = (obs info, with [5] being the thing obsoleted)
            # ugh

    def addSkipPatterns(self, skipPatterns):
        for p in skipPatterns:
            self.__skipPatternList.append(p)

    def getInstalledPackages(self):
        return self.__instPackagesHash

    def getForcedPackages(self):
        return self.__forcePackagesHash


    def getSkippedPackages(self):
        """
        Returns a list of packages that have been skipped, together with the
        reason why they've been skipped
        """
        # XXX
        return self.__skippedPackagesList

    def getObsoletedPackages(self):
        return self.__obsoletedPackagesList


    def getInstalledObsoletingPackages(self):
        return self.__installedObsoletingPackageList
    
    # list of packages we are going to install/update 
    # for ui stuff
    def getPackagesToInstall(self):
        db = rpmUtils.openrpmdb()
        # We put stuff in a hash to easily find packages
        hash = {}
        fpHash = self.__forcePackagesHash
        #print "self.__availPackagesList: %s" % self.__availPackagesList
        # Loop through the list of available packages
        for p in self.__availPackagesList:
            pName = p[0]

            # Is this package already installed?
            if not self.__instPackagesHash.has_key(pName):
                
                
                # Not installed. A good candidate for a kill
                # We now look for possible reasons we may want to install this
                # package
                # Is it in the list of forced packges?
                if not fpHash.has_key(pName):
                    # Not a forced package.
                    # Is this package obsoleting something already installed?
                    key = tuple(p[:5])
                    if not self.__obsolPackagesHash.has_key(key):
                        # This package doesn't obsolete anything, so skip it
                        continue
                    # This package obsoletes something
                    # Loop through the list of obsoletes
                    for obs in self.__obsolPackagesHash[key]:
                        obsName, obsVersion, obsSense = obs[5:]
                        # Is obsName installed?
                        if not self.__instPackagesHash.has_key(obsName):
                            # obsName not installed, so skip it
                            continue
                        # there is an installed package obsName
			if self.debug:
			    print "%s obsoletes %s" % (
                                obsName, self.__instPackagesHash[obsName])
                        if obsSense == 0:
                            # Obsoletes all versions. Add this package
                            break
                        # Have to compare versions
                        vv, rr, ee = parseObsoleteVersion(obsVersion)
                        candidate = (obsName, vv, rr, ee)
                        for inst in self.__instPackagesHash[obsName]:
                            ret = up2dateUtils.comparePackages(inst, candidate)
                            op = cmp2rpmSense(ret)
                            if op & int(obsSense):
                                # Gotcha.
                                break
                            else:
                                # the comparison between version and
                                # the currently installed packages
                                # failed, so skip this obsolete
                                continue
                        # If we got here (via a break) it means we have have
                        # to add the package
                        break
                    else: # for
                        # This package doesn't obsolete anything already
                        # installed, so skip it
                        continue
                    # Fall through
                else:
                    obsoletes = rpmUtils.getInstalledObsoletes()
                    if obsoletes.has_key(pName):
                        info = (p, obsoletes[pName] )
                        self.__installedObsoletingPackageList.append(info)
                        continue
                    # this is a package were "forcing" but we need to make sure
                    # it isnt obsoleted by something already installed
                    
            else:
                # heres a ugly one. If a package installed is
                # obsoleted by another package in the channel (or sub channel)
                # we dont really care if a newer versions is available or
                # not, since it's still gonna be obsoleted... so check to
                # see if something in the available package list obsoletes
                # this package, if so, dont bother updating it
                if self.__obsoletedPackageHash.has_key(p[0]):
                    self.__obsoletedPackagesList.append((p,self.__obsoletedPackageHash[p[0]]))
                    # this should really be smarter and do ver compares
                    continue
                # Installed
                # At this point we only care to find a newer package
		# but we have to check to verify that any of the versions
		# of the package arent up to date. So we iterate over the list
		# and set a flag if any of the installed packages are new
		# enough
		uptodate_flag = 0 
                for inst in self.__instPackagesHash[pName]:
		    #print "inst: %s p: %s " % (inst,p)
                    ret = up2dateUtils.comparePackages(inst, p)
                    if ret < 0:
			# the remote package is newer
			if self.debug:
			    print "p: %s is newer than inst: %s" % (p,inst) 
		    else:
			#indicates that at least on of the versions installed
			# is new enough
			uptodate_flag = 1
		    
                    # All the installed packages are newer or the same version
                    # with the remote one; skip it
		if uptodate_flag:
		    continue 
                    

            # This package has to be installed. Try to get the best matching
            # architecture
            archscore = rpm.archscore(p[4])
	    if self.debug:
		print "archscore: %s" %  archscore
            if archscore <= 0:
                # Unsupported architecture
                continue
            if not hash.has_key(pName):
                # The first time we see this package
                hash[pName] = p
                continue
            # The same version of the package was already present
            # Score the arches
            currentArch = hash[pName][4]
            if rpm.archscore(currentArch) < archscore:
                # We already have the better architecture
                continue
            # This is a better architecture
            hash[pName] = p

        # if were forcing packages, only include the packages
        # were forcing...
        # FIXME: I'm not a big fine of this, but its more
        # true to what you expect from this function
        if fpHash:
            tmpHash = {}
            for key in hash.keys():
                if fpHash.has_key(key):
                    tmpHash[key] = hash[key]
            hash = tmpHash
        # Prepare the result values
        result = hash.values()

        # Sort it just to make it look nice
        result.sort(lambda a, b: cmp(a[0], b[0]))
        result = self.__skipPackages(result)
        return result

    # list of failed deps if any
    # for ui stuff
    def failedDeps(self):
        pass

    # either return a rpm.TransactionSet object
    # or some similar representation
    def getTransaction(self):
        pass

    def __skipPackages(self, packageList):
        self.__skippedPackagesList = []

        for p_i in range(len(packageList)):
            for pattern in self.__skipPatternList:
                if fnmatch.fnmatch(packageList[p_i][0], pattern):
                    self.__skippedPackagesList.append((packageList[p_i], _("Pkg name/pattern")))

                    break

        for p in self.__skippedPackagesList:
            packageList.remove(p[0])

        packageList = self.__skipFiles(packageList)
        return packageList

    def __skipFiles(self, packageList):
        removedList = []
        if cfg.readEntry('forceInstall'):
            return packageList

        # need the db for the checkHeaderFiles
        db = rpmUtils.openrpmdb()
        
        p_i = 0
        h_list = buildHeaderList(packageList,msgCallback = self.msgCallback,
                                 progressCallback = self.progressCallback,
                                 refreshCallback = self.refreshCallback,
                                 ignoreMsgCallback = self.ignoreMsgCallback)

        for i in h_list:
            tmp_list = None
            tmp_list = rpmUtils.checkHeaderForFileConfigExcludes(i,packageList[p_i],db)
            p_i = p_i + 1
            if tmp_list:
   	         removedList = removedList + tmp_list


        d = {}
        for i in removedList:
            d[tuple(i[0])] = i
        removedList = d.values()

        for p in removedList:
            packageList.remove(p[0])

        self.__skippedPackagesList = self.__skippedPackagesList + removedList
        return packageList


def parseObsoleteVersion(ver):
    s = ver
    epoch = ""
    if string.find(s, ":") >= 0:
        arr = string.split(s, ":", 1)
        epoch = arr[0]
        s = arr[1]
    release = "0"
    if string.find(s, '-') >= 0:
        arr = string.split(s, "-", 1)
        release = arr[1]
        s = arr[0]
    return s, release, epoch
        

def cmp2rpmSense(value):
    if value < 0:
        return rpm.RPMSENSE_LESS
    if value > 0:
        return rpm.RPMSENSE_GREATER
    return rpm.RPMSENSE_EQUAL

def getArchesOfAvailablePackages(availablePkgList,pkgName):
    archList = []
    for pkg in availablePkgList:
        if pkgName == pkg[0]:
            archList.append(pkg[4])

    return archList

def buildHeaderList(pkgList,msgCallback = None, progressCallback = None,
                            refreshCallback = None, ignoreMsgCallback=None):
    # FIXME: Generalize
    # FIXME: plug in progressCallbacks
    h_list = []
    if msgCallback:
        msgCallback(_("Fetching rpm headers"))
        
    count = 1
    total = len(pkgList)

    for pkg in pkgList:
        hdr = headerList[pkg]
        if hdr:
            h_list.append(hdr)
        if msgCallback and not ignoreMsgCallback:
            msgCallback(_("Fetching rpm header: %s-%s-%s") % (hdr['name'],hdr['version'],hdr['release']))
        if progressCallback:
            progressCallback(count, total)
        count = count + 1
    return h_list
            

# side effects on pkgList
# REFACTOR: this probabaly needs to stay module scope, as
# depSolver.__skip calls it
def removeSkipPackagesFromList(pkgList, msgCallback = None,
                               progressCallback = None):
    removedList = []
    pkgCount = len(pkgList)

    if cfg.readEntry('forceInstall'):
        return removedList

    if msgCallback != None:
        msgCallback(_("Removing packages marked to skip from list"))

    pkgSkipList = cfg.readEntry("pkgSkipList")
    for index in range(pkgCount):
        for pattern in pkgSkipList:
            if fnmatch.fnmatch(pkgList[index][0], pattern):
                removedList.append((pkgList[index], _("Pkg name/pattern")))
                break

        if progressCallback != None:
            progressCallback(index+1, pkgCount)

    for pkg in removedList:
        pkgList.remove(pkg[0])

    # just do a side effect on pkgList here, and return removedList
    # since we actually need the results of it
    return removedList


if __name__ == '__main__':
    installedPackageList = []
    pList = PackageList()
    for i in range(20):
        installedPackageList.append("a%04d" % i, '1.0', '1', '', 'noarch')
    pList.addInstalledPackages(installedPackageList)
    availablePackageList = []
    for i in range(30):
        for arch in ['noarch', 'i386', 'i586', 'i686', 'athlon']:
            availablePackageList.append("a%04d" % i, '2.0', '1', '', arch)

    print len(installedPackageList), len(availablePackageList)
    pList.addAvailablePackages(availablePackageList)

    forcePackageList = []
    for i in range(10):
        forcePackageList.append("a%04d" % (20 + 5 * i))
    
#    pList.addForcePackages(forcePackageList)

    obsList = [
        ('a0021', '2.0', '1', '', 'i686', 'a0019', '', 0),
        ('a0022', '2.0', '1', '', 'i686', 'a0018', '2.0-1', 2),
    ]
    pList.addObsoletePackages(obsList)

    pList.addSkipPatterns(["a0017", "a001[3-5]"])
    
    import time
    start = time.time()
    res = pList.getPackagesToInstall()
    print "Total time: %f" % (time.time() - start)
    print len(res)
    print res
