#!/usr/bin/python
# rhnHTTPlib.py					     - Client-side HTTP logic.
#-----------------------------------------------------------------------------
# This is a library of HTTP-related functions for the client connection.
# A fair amount of code is shamelessly copied from cgiwrap and xmlrpclib.
#-----------------------------------------------------------------------------
#
# Copyright (c) 2001-2002 Red Hat, Inc.  Distributed under GPL
#
# Author: Mihai Ibanescu <misa@redhat.com>
# $Id: rhnHTTPlib.py,v 1.26 2002/04/10 23:06:06 alikins Exp $
"""Module of http based functions used by up2date. Mimics the interface
   of xmlrpclib"""

import os
import types
import string
import base64
import urllib
import httplib
from cgiwrap import myHTTP, myHTTPS, ProtocolError, Input, InputStream
from xmlrpclib import _Method

__version__ = "$Revision: 1.26 $"

class Transport:
    #'''
    #Handles an HTTP GET request to an RHN server.
    #'''
    user_agent = "rhnHTTPlib.py/%s" % string.split(__version__)[1]

    def __init__(self, http_type = "http", headers = {}):
        self.__lang = "C"
        if string.lower(http_type) not in ('http', 'https'):
            raise IOError, "Unknown HTTP scheme %s" % http_type
        self.__lang = "C"
        self.__ca_chain = None
        self.__headers = {}
        # Other headers we should initialize
        if headers:
            for (k, v) in headers.items():
                self.__headers[k] = v
        self.__type = string.lower(http_type)
        self.__httpobj = None
        #
        self.__host = None
        self.__proxy = None
        self.__handler = None
        self.__protocol = None


    def __del__(self):
        if self.__httpobj:
            self.__httpobj.close()
        self.close()
    

    # Add/replace additional arbitrary header.
    def add_header(self, name, arg):
	self.__headers[name] = arg


    def request(self, host, proxy, handler, req, username = None, 
                password = None, protocol = 'HTTP/1.0'):
        # Function that issues a GET
        # It tries to use the same connection if possible

        protocol = string.upper(protocol)
        if protocol[:5] != 'HTTP/': protocol = 'HTTP/1.0'
        # Set the protocol version
        protocolVersion = protocol[5:]
        # Try to convert it to an int
        try:
            protocolVersion = int(10*float(protocolVersion))
        except:
            # Assume HTTP/1.0
            protocolVersion = 10

        if not self.__httpobj or self.__host != host or \
                self.__proxy != proxy or self.__handler != handler or \
                self.__protocol != protocol:
            self.__host = host
            self.__proxy = proxy
            self.__handler = handler
            self.__protocol = protocol
            # Set header fields
            self.__headers["User-Agent"] = self.user_agent
            self.__headers["X-Info"] = \
              "Red Hat Network Update Agent (C) Red Hat, Inc "+\
              "($Revision: 1.26 $)"
            if username and password:
                self.__username = username
                self.__password = password
                userpass = "%s:%s" % (self.__username, self.__password)
                enc_userpass = string.strip(base64.encodestring(userpass))
                self.__headers["Proxy-Authorization"] = " Basic %s"\
                                                        % enc_userpass
            if not self.__lang == "C":
                self.__headers["Accept-Language"] = self.__lang
            _proto = string.lower(self.__type)
            if proxy:
                _proxy, _port = urllib.splitnport(proxy)
                if _port is None:
                    _port = 0
                if _proto == 'http':
                    h = myHTTP(_proxy, _port, protocol)
                else:
                    h = myHTTPS(_proxy, _port, self.__ca_chain, host,
                        username=username, password=password,
                        protocol=protocol)
            else:
                if _proto == 'http':
                    h = myHTTP(host, 0, protocol)
                else:
                    h = myHTTPS(host, 0, self.__ca_chain, protocol = protocol)
            # Set the local __httpobj
            self.__httpobj = h
        else:
            # We already have the connection
            h = self.__httpobj
        httplib.HTTP_VERSION = protocol
        h.putrequest("GET", "%s/%s" % (handler, req))
        # Send headers
        h.putheader("Host", host)
        # XXX Add a Date field
        for (k, v) in self.__headers.items():
            if type(v) in (types.ListType, types.TupleType):
                for vv in v:
                    h.putheader(k, str(vv))
            else:
                h.putheader(k, str(v))
        h.endheaders()
        errcode, errmsg, headers = h.getreply()

        # FIXME: what errmsg/errcode should I raise here?
        # basically, the server has abrubtly closed my connection...
        if not headers:
            errmsg = "The server appears to have closed this connection"
            if proxy:
                raise ProtocolError(proxy + " " + handler, errcode,
                                    errmsg, headers)
            else:
                raise ProtocolError(host + " " + handler, errcode,
                                    errmsg, headers)
        if protocolVersion <= 10:
            # Versions prior to or HTTP/1.0
            if not headers.has_key("Keep-Alive"):
                # Next time open a different connection
                self.__httpobj = None
        elif headers.has_key("Connection") and \
                headers["Connection"] == "close":
            self.__httpobj = None
        fd = h.getfile()
        
        if errcode != 200:
            fd.close()
            errmsg = "%s: \"%s\" while attempting to get \n%s" % (errcode, errmsg, req)
            if proxy:
                raise ProtocolError(proxy + " " + handler, errcode, errmsg, 
                    headers)
            else:
                raise ProtocolError(host + " " + handler, errcode, errmsg,
                    headers)

        return headers, fd


    def setlang(self, lang):
        self.__lang = lang


    def use_CA_chain(self, ca_chain = None):
        if not ca_chain:
            return 0
        if not os.access(ca_chain, os.R_OK):
            raise IOError, "the CA cert %s was not accessible" % ca_chain
        self.__ca_chain = ca_chain
        return 1


    def close(self):
        if self.__httpobj:
            # Don't close the http obj, since that closes the file descriptor
            # too, and we may still need it.
            self.__httpobj = None
        

    def __repr__(self):
        return "<GET Transport class>"
    __str__ = __repr__

class Server:
    #'''
    #Represents a connection to an RHN server serving requests via GET
    #'''
    def __init__(self, uri, proxy = None, transport = None, username = None, 
                password = None, clientVersion = 2,
		headers = {}, protocol = 'HTTP/1.0'):
        # get the url
        type, uri = urllib.splittype(uri)
        if not string.lower(type) in ["http", "https"]:
            raise IOError, "unsupported protocol"
        self.__type = type
        self.__host = urllib.splithost(uri)[0]
        if proxy:
            self.__proxy = proxy
        else:
            self.__proxy = None

        self.__handler = type + ":" + uri
        
        self.__headers = headers
	self.__headers['X-RHN-Client-Version'] = str(clientVersion)
        self.__transport = transport
        self.__lang = "C"
        self.__username = username
        self.__password = password
        self.__ca_chain = None
        self.__protocol = string.upper(protocol)


    def __del__(self):
        self.close()

    def __request(self, methodname, params):
        '''
        RET: size: size of body (0 if RHN Fault or other problem).
               fd: socket file descriptor *OR* if RHN Fault, the headers.
        '''
        # Check the params
        if not params or len(params) < 1:
            # We need at least the channel name
            # XXX Find an rhnFault for this
            raise Exception("Channel is not present")
            
        if not self.__transport:
            self.__transport = Transport(headers = self.__headers, 
					http_type=self.__type)
            if hasattr(self.__transport, "setlang"):
                self.__transport.setlang(self.__lang)
            if hasattr(self.__transport, "use_CA_chain"):
                self.__transport.use_CA_chain(self.__ca_chain)

        # Compose the URL to be sent
        url = "$RHN/%s/%s" % (params[0], methodname)
        # Strip the channel name from the params
        args = (url, ) + params[1:]
        url = string.join(args, '/')

        headers, fd = self.__transport.request(
            self.__host,
            self.__proxy,
            self.__handler,
            url,
            self.__username,
            self.__password,
            self.__protocol)

        if 0:
            print "THE HEADERS COMING IN (rhnHTTPlib.py):"
            for each in headers.keys():
                print "\t%s : %s" % (each, headers[each])

        # At this point we have received a 200 (anything else raises
        # exceptions)

        # Create the input object
        input = Input(headers)
        try:
            fd = input.decode(fd)
        except InputStream, e:
            return e.length, e.fd

        l = input.length
        del input
        return l, fd
            
    # Allow user-defined additional headers (name, arg).
    def add_header(self, name, arg):
	if self.__transport:
	    self.__transport.add_header(name, arg)
 
    # Allow user-defined additional headers (dictionary).
    def add_headers(self, dict):
	if self.__transport and type(dict) == types.TupleType:
            for k in dict.keys():
	        self.__transport.add_header(k, dict[k])
 
    def set_server(self, protocol):
        self.__protocol = string.upper(protocol)
    
    def use_CA_chain(self, ca_chain = None):
        self.__ca_chain = ca_chain


    def __getattr__(self, name):
        return _Method(self.__request, name)

    def close(self):
        if self.__transport:
            self.__transport.close()
            self.__transport = None


    def __repr__(self):
        return "<GET Server class>"
    __str__ = __repr__

def reportError(headers):
    # Reports the error from the headers
    errcode = 0
    errmsg = ""
    s = "X-RHN-Fault-Code"
    if headers.has_key(s):
        errcode = int(headers[s])
    s = "X-RHN-Fault-String"
    if headers.has_key(s):
        _sList = headers.getHeaderValues(s)
        if _sList:
            _s = string.join(_sList, '')
            errmsg = "%s" % base64.decodestring(_s)

    return errcode, errmsg
            
#            print "(rhnHTTPlib.py) STUB CODE: RHN-Fault occured!!!: %s : "\
#                % headers["X-RHN-Fault-Code"],
#                else: print
#            return 0, headers

#-----------------------------------------------------------------------------
if __name__ == "__main__":
    print "You can not run this module by itself"
    import sys; sys.exit(-1)
#-----------------------------------------------------------------------------

