#!/usr/bin/python
import os
import sys
import cgiwrap
import config
import up2dateLog
import up2dateErrors 
import up2dateAuth
import up2dateUtils 
import wrapperUtils

import socket
import string
import rhnHTTPlib
import time

# so we can use the same code with python 1.5.2 and 2.2
try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")
    
from translate import _, N_, cat
            
# get a reference to the log file object
log = up2dateLog.initLog()
            

#grab an instance of the config class shared
cfg = config.initUp2dateConfig()

warningCallback = wrapperUtils.warningCallback

# Where do we keep the CA certificate for RHNS?
# The servers we're talking to need to have their certs
# signed by one of these CA.
if cfg.readEntry("sslCACert"):
    rhns_ca_cert = cfg.readEntry("sslCACert")
else:
    rhns_ca_cert = "/usr/share/rhn/RHNS-CA-CERT"

def hasSSL():
    return hasattr(socket, "ssl")

def getServer(refreshCallback=None):
    global warningCallback

        
    if cfg.readEntry("enableProxy"):
        proxyHost = up2dateUtils.getProxySetting()
    else:
        proxyHost = None

    if hasSSL():
        serverUrl = cfg.readEntry("serverURL")
    else:
        serverUrl = cfg.readEntry("noSSLServerURL")

    proxyUser = None
    proxyPassword = None
    if cfg.readEntry("enableProxyAuth"):
        proxyUser = cfg.readEntry("proxyUser") or None
        proxyPassword = cfg.readEntry("proxyPassword") or None

    lang = None
    for env in 'LANGUAGE', 'LC_ALL', 'LC_MESSAGES', 'LANG':
        if os.environ.has_key(env):
            if not os.environ[env]:
                # sometimes unset
                continue
            lang = string.split(os.environ[env], ':')[0]
            lang = string.split(lang, '.')[0]
            break

        
    s = rpclib.Server(serverUrl, proxyHost,
			 transport=None,
                         refreshCallback=refreshCallback,
                         username=proxyUser,
                         password=proxyPassword)
    if lang:
        s.setlang(lang)

    # require RHNS-CA-CERT file to be able to authenticate the SSL connections
    if not os.access(rhns_ca_cert, os.R_OK):
        if warningCallback:
            warningCallback("ERROR: can not find RHNS CA file: %s" % rhns_ca_cert)
        log.log_me("ERROR: can not find RHNS CA file: %s" % rhns_ca_cert)
        sys.exit(-1)

    # force the validation of the SSL cert
    s.use_CA_chain(rhns_ca_cert)    

    return s


# FIXME: doCall should probabaly be a method
# of a higher level server object
def doCall(method, *args, **kwargs):
    ret = None

    attempt_count = 1
    if cfg.readEntry("networkRetries"):
        attempts = cfg.readEntry("networkRetries")
    else:
        attempts = 5
    
    while 1:
        failure = 0
        ret = None
        try:
            ret = apply(method, args, kwargs)
        except KeyboardInterrupt:
            raise up2dateErrors.CommunicationError(_("Connection aborted by the user"))
        # if we get a socket error, keep trying
        except (socket.error, socket.sslerror), e:
            log.log_me("A socket error occurred: %s, attempt #%s" % (e, attempt_count))
            if attempt_count >= attempts:
                if len(e.args) > 1:
                    raise up2dateErrors.CommunicationError(e.args[1])
                else:
                    raise up2dateErrors.CommunicationError(e.args[0])
            else:
                failure = 1
        except cgiwrap.ProtocolError, e:
            log.log_me("A protocol error occurred: %s , attempt #%s" % (e.errmsg, attempt_count))
            (errCode, errMsg ) = rhnHTTPlib.reportError(e.headers)
            reset = 0
            if abs(errCode) == 34:
                log.log_me("Auth token timeout occurred\n errmsg: %s" % errMsg)
                # this calls login, which in tern calls doCall (ie,
                # this function) but login should never get a 34, so
                # should be safe from recursion

                # FIXME: move to up2dateAuth
                up2dateAuth.updateLoginInfo()
                reset = 1

            # the servers are being throttle to pay users only, catch the
            # exceptions and display a nice error message
            if abs(errCode) == 51:
                log.log_me(_("Server was refused connection due to high load"))
                raise up2dateErrors.CommunicationError(e.errmsg)


            # if we get a 404 from our server, thats pretty fatal... no point in
            # retrying over and over. Note that errCode == 17 is specific to
            # our servers, if the serverURL is just pointing somewhere random
            # they will get a 0 for errcode and will raise a CommunicationError
            if abs(errCode) == 17:
                log.log_me("File Not Found: %s" % errMsg)
                raise up2dateErrors.FileNotFoundError(errMsg)
                
            if not reset:
                if attempt_count >= attempts:
                    raise up2dateErrors.CommunicationError(e.errmsg)
                else:
                    failure = 1
            
        except rpclib.ResponseError:
            raise up2dateErrors.CommunicationError("Broken response from the server.");

        if failure:
            # rest for five seconds before trying again
            time.sleep(5)
            attempt_count = attempt_count + 1
        if ret != None:
            break
        
    return ret
    
