#!/usr/bin/python
#
# a chain of responsibilty class for stacking package sources
# for up2date
# Copyright (c) 1999-2002 Red Hat, Inc.  Distributed under GPL.
#
# Author: Adrian Likins <alikins@redhat.com>
#
"""A chain of responsibility class for stacking package sources for up2date"""

import up2dateUtils
import up2dateLog
import up2dateErrors
import glob
import socket
import xmlrpclib, cgiwrap
import re
import os
import rpm
import string
import time
import struct
import config
import rpmUtils
import up2dateAuth
import up2dateUtils
import headers

from translate import _, N_, cat
import rhnHTTPlib

cfg = config.initUp2dateConfig()
log = up2dateLog.initLog()

import up2date
try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")


BUFFER_SIZE = 8092

DEBUG=0
def DPRINT(string):
    if DEBUG:
        print  string



# containment class for handling server config info
class ServerSettings:
    def __init__(self):
        self.xmlrpcServerUrl = cfg.readEntry("serverURL")
	if cfg.readEntry("useNoSSLForPackages"):
            self.httpServerUrl = cfg.readEntry("noSSLServerURL")
	else:
	    self.httpServerUrl = cfg.readEntry("serverURL")

        self.proxyUrl = None
        self.proxyUser = None
        self.proxyPassword = None
        
        if cfg.readEntry("enableProxy") and up2dateUtils.getProxySetting():
            self.proxyUrl = up2dateUtils.getProxySetting()
            if cfg.readEntry("enableProxyAuth"):
                if cfg.readEntry("proxyUser") and cfg.readEntry("proxyPassword"):
                    self.proxyPassword = cfg.readEntry("proxyPassword")
                    self.proxyUser = cfg.readEntry("proxyUser")
                    
    def settings(self):
        return self.xmlrpcServerUrl,self.httpServerUrl, self.proxyUrl, self.proxyUser, self.proxyPassword


# when intilized, the class doesnt have any info
# but "updateLoginInfo" will call it...
# we just need one instance of this class, at module scope
class PackageSourceConfig:
    def __init__(self):
        self.headerSource = None
        self.packageSource = None
        self.sourcePackageSource = None
        self.listPackageSource = None
        self.getObsoletesSource = None
        self.serverSettings = ServerSettings()

    # initalize the sources to start with, but only
    # update them otherwise
    def updateLoginInfo(self, logininfo):
        if self.headerSource and self.packageSource \
               and self.sourcePackageSource and self.listPackageSource:
            self.updatePackageSource(logininfo)
        else:
            self.initPackageSource(logininfo)

    # FIXME:  this is ugly and stupid, but it works...
    # the "updateHttpServer" thing is pretty lame and ugly
    def updatePackageSource(self, logininfo):
        self.headerSource = updateHttpServer(self.headerSource, logininfo, self.serverSettings)
        self.packageSource = updateHttpServer(self.packageSource, logininfo, self.serverSettings)
        self.sourcePackageSource = updateHttpServer(self.sourcePackageSource, logininfo, self.serverSettings)
        self.listPackageSource = updateHttpServer(self.listPackageSource, logininfo, self.serverSettings)
        self.getObsoletesSource = updateHttpServer(self.getObsoletesSource,  logininfo, self.serverSettings)

    def initPackageSource(self, logininfo):
        self.headerSource = initHeaderSources(logininfo, self.serverSettings)
        self.packageSource = initPackageSources(logininfo, self.serverSettings)
        self.sourcePackageSource = initSourcePackageSources(logininfo, self.serverSettings)
        self.listPackageSource = initListPackageSources(logininfo, self.serverSettings)
        self.getObsoletesSource = initGetObsoletesSources(logininfo, self.serverSettings)
        

# this instance holds all the info about the package
# source objects
psc = PackageSourceConfig()

# FIXME: this is duplicated all over the place
def pkgToString(pkg):
    return "%s-%s-%s" % (pkg[0], pkg[1], pkg[2])

def pkgToStringArch(pkg):
    return "%s-%s-%s.%s" % (pkg[0], pkg[1], pkg[2], pkg[4])

def factory(aClass, *args, **kwargs):
    return apply(aClass, args, kwargs)


def saveHeader(hdr):
    fileName = "%s/%s.%s.hdr" % (cfg.readEntry("storageDir"),
                              string.join((hdr['name'],
                                           hdr['version'],
                                           hdr['release']), "-"), hdr['arch'])

    fd = os.open(fileName, os.O_WRONLY|os.O_CREAT, 0600)

    os.write(fd, hdr.unload())
    os.close(fd)

    return 1


def readHeader(filename):
    if not os.access(filename, os.R_OK):
        return None
    blob = open(filename, "r").read()
    return readHeaderBlob(blob)

def readHeaderBlob(blob):
    # Read two unsigned int32
    i0, i1 = struct.unpack("!2I", blob[:8])
    if len(blob) != i0 * 16 + i1 + 8:
        # Corrupt header
        return None
    # if this header is corrupt, rpmlib exits and we stop ;-<
    try:
        hdr = rpm.headerLoad(blob)
    except:
        return None
    # Header successfully read
    return hdr

# this is going to be a factory. More than likely, it's input for
# it's init is going to be read from a config file. 
#
#  goals... easy "stacking" of data sources with some sort of priority
#      make the goal oblivous to the "names" of the data sources
# 
class PackageSourceChain:
    def __init__(self, headerCacheObject = None, metainfolist = None):
        self.metainfo = {}
        # lame, need to keep a list to keep the order
        self.source_list = []
        if metainfolist != None:
            for source in metainfolist:
                self.addSourceClass(source)

        self.headerCache = headerCacheObject


    def addSourceClass(self, metainfo):
        source = metainfo
        name = source['name']
        DPRINT("class name: %s" % name)
        self.source_list.append(name)
        self.metainfo[name] = factory(source['class'],
                                      source['args'],
                                      source['kargs'])

        # automagically associated the header cache for the stack with each
        # object in the chain so we can populate the cache if need be
        self.metainfo[name].addHeaderCacheObject(self.headerCache)
        DPRINT("class: %s args: %s kargs %s" % (source['class'], source['args'], source['kargs']))


    def addSourceInstance(self, metainfo):
        source = metainfo
        name = source['name']
        DPRINT("class name: %s" % name)
        self.source_list.append(name)
        self.metainfo[name] = source['object']

        # automagically associated the header cache for the stack with each
        # object in the chain so we can populate the cache if need be
        self.metainfo[name].addHeaderCacheObject(self.headerCache)
        self.metainfo[name]['name'] =  name
        
        DPRINT("object: %s name: %s" % (source['object'], name))

    def getPackage(self, pkg, MsgCallback = None, progressCallback = None):
        DPRINT(len(pkg))
        for source_key in self.source_list:
            source = self.metainfo[source_key]
            # FIXME: wrap in approriate exceptions
            package = source.getPackage(pkg, MsgCallback, progressCallback)
            if package != None:
                DPRINT("\n\n*** Package Fetched via : %s *** \n\n" % source['name'])
                #self.fetchType[pkg] = source['name']
                return package
        return None

    def getPackageSource(self, channel, pkg, MsgCallback = None, progressCallback = None):
        DPRINT(len(pkg))
        for source_key in self.source_list:
            source = self.metainfo[source_key]
            DPRINT(source)
            DPRINT(dir(source))
            # FIXME: wrap in approriate exceptions
            package = source.getPackageSource(channel, pkg, MsgCallback, progressCallback)
            if package != None:
                DPRINT("\n\n*** Source Package Fetched via : %s *** \n\n" % source['name'])
                return package
        return None
    

    def getHeader(self, pkg, msgCallback = None, progressCallback = None):
        # if the package source is specified, use it
        for source_key in self.source_list:
            source = self.metainfo[source_key]
            header = source.getHeader(pkg, progressCallback = progressCallback)
            # return the first one we find
            if header != None:
                DPRINT("\nHeader Fetched via: %s\n" % source['name'])
                # FIXME, the normal one only returns the header
                return (header,source['name'])
        return None

    def listPackages(self, channel, version, msgCallback = None, progressCallback = None):
        
        for source_key in self.source_list:
            source = self.metainfo[source_key]
            channelList = source.listPackages(channel, version, msgCallback, progressCallback)
            if channelList != None:
                DPRINT("\n\nlistPackages Fetched via: %s:\n" % source['name'])
                return (channelList, source['name'])
        return None


    def getObsoletes(self, obsoletes, version, msgCallback = None, progressCallback = None):
        for source_key in self.source_list:
            source = self.metainfo[source_key]
            obsoletesList = source.getObsoletes(obsoletes, version, msgCallback, progressCallback)
            if obsoletesList != None:
                up2dateUtils.DPRINT("\n\ngetObsoletes Fetched via: %s:\n" % source['name'])
                return (obsoletesList, source['name'])
        return None


# template class for all the sources in the chain
class PackageSource:
    def __init__(self, cacheObject = None):
        self.headerCache = None
        self.info = {}

    def getHeader(self, pkg):
        pass

    def getPackage(self, pkg):
        pass

    def addHeaderCacheObject(self, cacheObject):
        self.headerCache = cacheObject

    def __setitem__(self, item, value):
        self.info[item] = value

    def __getitem__(self, item):
        return self.info[item]
        
class HeaderMemoryCache(PackageSource):
    def __init__(self, cacheObject = None):
        PackageSource.__init__(self,cacheObject)
    
    def getHeader(self, pkg, lite = None, msgCallback = None, progressCallback = None):
        if lite:
            if self.headerCache.has_key(pkgToStringArch(pkg), lite = 1):
                return self.headerCache.getLite(pkgToStringArch(pkg))

        if self.headerCache.has_key(pkgToStringArch(pkg)):
            return self.headerCache[pkgToStringArch(pkg)]


class LocalDisk(PackageSource):
    def __init__(self, cacheObject = None, packagePath = None):
        self.dir_list = up2dateUtils.getPackageSearchPath()
        if packagePath:
            self.dir_list = self.dir_list + packagePath
        else:
            # use the default appended package dirs
            if cfg.readEntry("packageDir"):
                self.dir_list = self.dir_list + string.split(cfg.readEntry("packageDir"),':')
        
        PackageSource.__init__(self, cacheObject = cacheObject)

    def addPackageDir(self, packagePath):
        self.dir_list = self.dir_list + packagePath

    def __saveHeader(self, hdr):
        tmp = readHeaderBlob(hdr.unload())
        saveHeader(tmp)
        

    def getHeader(self, pkg, msgCallback = None, progressCallback = None):
        baseFileName = "%s-%s-%s.%s.rpm" % (pkg[0], pkg[1], pkg[2], pkg[4])
        for dir in self.dir_list:
            tmpFileNames = glob.glob("%s/%s" % (dir, baseFileName))
            fileName = "%s/%s" % (cfg.readEntry("storageDir"), baseFileName)

        fileNames = tmpFileNames
        if len(fileNames):
            if os.access(fileNames[0], os.R_OK):
                if not re.search("rpm$", fileNames[0]):
                    # it wasnt an rpm, so must be a header
                    if os.stat(fileNames[0])[6] == 0:
                        return None
                    fd = open(fileNames[0], "r")
                    # if this header is corrupt, rpmlib exits and we stop ;-<
                    try:
                        hdr = readHeaderBlob(fd.read())
                    except:
                        return None
                    self.headerCache[pkgToStringArch(pkg)] = hdr
                    fd.close()
                    self.__saveHeader(hdr)
                    return hdr
                else:
                    if not rpmUtils.checkRpmMd5(fileNames[0]):
                        fd = os.open(fileNames[0], 0)
                        try:
                            (hdr,isSource) = rpm.headerFromPackage(fd)
                        except:
                            os.close(fd)
                            raise up2dateErrors.RpmError(_("Error reading header"))
                        os.close(fd)
                        self.headerCache[pkgToStringArch(pkg)] = hdr
                        self.__saveHeader(hdr)
                        return hdr
                    else:
                        return None
            else:
                 return None        
        else:
            for dir in self.dir_list:
                fileNames = glob.glob("%s/%s.noarch.*" %
                                      (dir,
                                       pkgToString(pkg)))
            if len(fileNames):
                if os.access(fileNames[0], os.R_OK):
                    if not re.search("rpm$", fileNames[0]):
                        # it's not an rpm, must be a header
                        if os.stat(fileNames[0])[6] == 0:
                            return None
                        fd = open(fileNames[0], "r")
                        try:
                            hdr = readHeaderBlob(fd.read())
                        except:
                            log.log_me("Corrupt header %s, skipping, will download later..." % fileNames[0])
                            return None
                        fd.close()
                        self.headerCache[pkgToStringArch(pkg)] = hdr
                        return hdr
                    else:
                        if not rpmUtils.checkRpmMd5(fileNames[0]):
                            fd = os.open(fileNames[0], 0)
                            try:
                                (hdr,isSource) = rpm.headerFromPackage(fd)
                            except:
                                os.close(fd)
                                raise up2dateErrors.RpmError(_("Error reading header"))
                            os.close(fd)
                            log.log_me("Reading header from: %s" % fileNames)
                            self.headerCache[pkgToStringArch(pkg)] = hdr
                            return hdr
                        else:
                            return None
                else:
                    return None
            else:
                return None
    
    # this is kind of odd, since we actually just symlink to the package from
    # the cache dir, instead of copying it around, or keeping track of where
    # all the packages are from
    def getPackage(self, pkg, msgCallback = None, progressCallback = None):
        baseFileName = "%s-%s-%s.%s.rpm" % (pkg[0], pkg[1], pkg[2], pkg[4])
        for dir in self.dir_list:
            tmpFileNames = glob.glob("%s/%s" % (dir, baseFileName))
            fileName = "%s/%s" % (cfg.readEntry("storageDir"), baseFileName)
            # if the file is in the storageDir, dont symlink it to itself ;->
            if len(tmpFileNames) and not (dir == cfg.readEntry("storageDir")):
                try:
                    os.remove(fileName)
                except OSError:
                    pass
                if msgCallback:
                    msgCallback(fileName)
                os.symlink(tmpFileNames[0], fileName)
                break

            
    def getPackageSource(self, channel, srcpkg, msgCallback = None, progressCallback = None):
        baseFileName = "%s" % (srcpkg)
        for dir in self.dir_list:
            tmpFileNames = glob.glob("%s/%s" % (dir, baseFileName))
            fileName = "%s/%s" % (cfg.readEntry("storageDir"), baseFileName)
            # if the file is in the storageDir, dont symlink it to itself ;->
            if len(tmpFileNames) and not (dir == cfg.readEntry("storageDir")):
                try:
                    os.remove(fileName)
                except OSError:
                    # symlink doesnt exist, this is fine
                    pass
                if msgCallback:
                    msgCallback(fileName)
                os.symlink(tmpFileNames[0], fileName)
                break


class DiskCache(PackageSource):
    def __init__(self, cacheObject = None):
        self.dir_list = up2dateUtils.getPackageSearchPath()
            
        PackageSource.__init__(self, cacheObject = cacheObject)

    def __readHeaderFromRpm(self, fileNames, pkg):
         if not rpmUtils.checkRpmMd5(fileNames[0]):
             fd = os.open(fileNames[0], 0)
             try:
                 (hdr,isSource) = rpm.headerFromPackage(fd)
             except:
                 os.close(fd)
                 raise up2dateErrors.RpmError(_("Error reading header"))
             os.close(fd)
             log.log_me("Reading header from: %s" % fileNames)
             self.headerCache[pkgToStringArch(pkg)] = hdr
             return hdr
         else:
             return None

    def __readHeaderFromFile(self, fileNames, pkg):
        if os.access(fileNames[0], os.R_OK):
            if os.stat(fileNames[0])[6] == 0:
                print "stat failed"
                return None
            hdr = readHeader(fileNames[0])
            if hdr == None:
                return None
            self.headerCache[pkgToStringArch(pkg)] = hdr
            return hdr
        else:
            return None
        
    def getHeader(self, pkg, msgCallback = None, progressCallback = None):
        for dir in self.dir_list:
            fileNames = glob.glob("%s/%s.%s.hdr" %
                                  (dir,
                                   pkgToString(pkg),
                                   pkg[4]) )
            # if we find anything, bail
            if len(fileNames):
                break


        if len(fileNames):
            hdr = self.__readHeaderFromFile(fileNames, pkg)
            if hdr:
                return hdr
            else:
                return None

        else:
            for dir in self.dir_list:
                fileNames = glob.glob("%s/%s.noarch.hdr" %
                                      (dir,
                                       pkgToString(pkg)))

            # see if it is a .hdr file, if not try reading it as an rpm
            if len(fileNames):
                hdr = self.__readHeaderFromFile(fileNames, pkg)
                if hdr:
                    return hdr
                else:
                    hdr = self.__readHeaderFromRpm(fileNames,pkg)
                    return hdr

    def getPackage(self, pkg, msgCallback = None, progressCallback = None):

        baseFileName = "%s-%s-%s.%s.rpm" % (pkg[0], pkg[1], pkg[2], pkg[4])
        # check if we already have package and that they are valid
        fileName = "%s/%s" % (cfg.readEntry("storageDir"),
                              baseFileName)

        if os.access(fileName, os.R_OK) and not rpmUtils.checkRpmMd5(fileName):
            if msgCallback:
                msgCallback(baseFileName)
            if progressCallback != None:
                progressCallback(1, 1)
            return 1
        else:
            return None

    def getPackageSource(self, channel, srcpkg, msgCallback = None, progressCallback = None):
        baseFileName = "%s" % (srcpkg)
        fileName = "%s/%s" % (cfg.readEntry("storageDir"),
                              baseFileName)
        
        # check if we already have package and that they are valid
        if os.access(fileName, os.R_OK):
#        if os.access(fileName, os.R_OK) and not rpmUtils.checkRpmMd5(fileName):
            if msgCallback:
                msgCallback(baseFileName)
            if progressCallback != None:
                progressCallback(1, 1)
            return 1
        else:
            return None


    def listPackages(self, channel, version, msgCallback = None, progressCallback = None):
        localFilename = "%s/%s.%s" % (cfg.readEntry("storageDir"), channel, version)
        if not os.access(localFilename, os.R_OK):
            return None

        # FIXME error handling and what not
        f = open(localFilename, "r")
        filecontents = f.read()
        # bump it full
        if progressCallback:
            progressCallback(100,100)
        
        try:
            tmp_args, tmp_method = xmlrpclib.loads(filecontents)
        except:
            # if there was an error decoding return as if we didnt find it
	    # the odd thing is, in testing, it's actually pretty hard to get
	    # a file to show as corrupt, I think perhaps the xmlrpclib parser
	    # is a bit too lenient...
	    return None

        # tmp_args[0] is the list of packages
        return tmp_args[0]


    def getObsoletes(self, obsoletes, version, msgCallback = None, progressCallback = None):
        localFilename = "%s/%s-obsoletes.%s" % (cfg.readEntry("storageDir"), obsoletes, version)
        if not os.access(localFilename, os.R_OK):
            return None

        # FIXME error handling and what not
        f = open(localFilename, "r")
        filecontents = f.read()
        # bump it full
        if progressCallback:
            progressCallback(100,100)

        try:
            tmp_args, tmp_method = xmlrpclib.loads(filecontents)
        except:
            # if there was an error decoding return as if we didnt find it
	    # the odd thing is, in testing, it's actually pretty hard to get
	    # a file to show as corrupt, I think perhaps the xmlrpclib parser
	    # is a bit too lenient...
	    return None

        # tmp_args[0] is the list of packages
        return tmp_args[0]
        
class HttpGetSource(PackageSource):
    def __init__(self, server, proxyHost, loginInfo = None, cacheObject = None):
        self.s = server
        PackageSource.__init__(self, cacheObject = cacheObject)
        self._loginInfo=loginInfo

    def _readFD(self, fd, size, filename, progressCallback = None):
        # Open the storage file
        f = open(filename, "w+")
        startTime = time.time()
        lastTime = startTime
        origsize = size
        while size > 0:
            curTime = time.time()
            l = min(BUFFER_SIZE, size)
            chunk = fd.read(l)
            # read guarantees that len(chunk) <= l
            l = len(chunk)
            if not l:
                # Nothing more to read
                break
            # And since the original l was smaller than size, we know 
            # now size >= 0
            size = size - l
            f.write(chunk)
            # Is it time to update the progress bar?
            if curTime - lastTime >= 1 or size == 0:
                lastTime = curTime
                bytesRead = origsize - size
                # a bitty epsilon to prevent devide by zero
                speed = bytesRead / ((curTime - startTime) + .000001)
                if speed != 0:
                    if origsize == 0:
                        secs = 0
                    else:
                        secs = size / speed
#                print "br: %s os: %s speed: %s secs: %s" % (bytesRead, origsize, speed, secs)
                if progressCallback != None:
                    progressCallback(bytesRead, origsize, speed, secs)

        if size > 0:
            # Probably the connection timed out
            f.close()
            os.unlink(filename)
            raise up2dateErrors.CommunicationError("There was a network error. It appears the server has abruplty closed the connection")
        # Flush the file to disk, then read it
        f.flush()
        # Rewind
        f.seek(0, 0)
        return f.read()
            

    def getHeader(self, package, msgCallback = None, progressCallback = None):
        hdr = None
        # package list format
        # 0        1        3       4     5     6      7
        # name, version, release, epoch, arch, size, channel

        filename = "%s-%s-%s.%s.hdr" % (package[0], package[1], package[2],
            package[4])
        channel = package[6]

        filePath = "%s/%s" % (cfg.readEntry('storageDir'), filename)
        size, fd = self.s.getPackageHeader(channel, filename)

        pkgname = "%s-%s-%s" % (package[0], package[1], package[2])
        if msgCallback:
            msgCallback(pkgname)

        buffer = self._readFD(fd, size, filePath, progressCallback=progressCallback)
        fd.close()

        hdr = readHeaderBlob(buffer)
        saveHeader(hdr)
        self.headerCache["%s-%s-%s.%s" % (hdr['name'],
                                       hdr['version'],
                                       hdr['release'],
                                       hdr['arch'])] = hdr
        return hdr

    
    def getPackage(self, package, msgCallback = None, progressCallback = None):
        filename = "%s-%s-%s.%s.rpm" % (package[0], package[1], package[2],
            package[4])
        channel = package[6]
        
        filePath = "%s/%s" % (cfg.readEntry("storageDir"), filename)
        size, fd = self.s.getPackage(channel, filename)
       
        if msgCallback:
            msgCallback(filename)
            
        buffer = self._readFD(fd, size, filePath, progressCallback = progressCallback)
        fd.close()
        return buffer

    
    def getPackageSource(self, channel, package, msgCallback = None, progressCallback = None):
        filename = package

        filePath = "%s/%s" % (cfg.readEntry("storageDir"), filename)
        size, fd = self.s.getPackageSource(channel, filename)

        if msgCallback:
            msgCallback(package)

        buffer = self._readFD(fd, size, filePath, progressCallback = progressCallback)
        fd.close()
        return buffer
        

    def listPackages(self, channel, version, msgCallback = None, progressCallback = None):
        filePath = "%s/%s.%s" % (cfg.readEntry("storageDir"), channel, version)
        size, fd = self.s.listPackages(channel, version)

        buffer = self._readFD(fd, size, filePath, progressCallback=progressCallback)
        fd.close()

        try:
            tmp_args, tmp_method = xmlrpclib.loads(buffer)
        except:
            # if there was an error decoding return as if we didnt find it
            return None
        
        # tmp_args[0] is the list of packages
        return tmp_args[0]

    def getObsoletes(self, obsoletes, version, msgCallback = None, progressCallback = None):
        filePath = "%s/%s-obsoletes.%s" % (cfg.readEntry("storageDir"), obsoletes, version)
        size,fd = self.s.getObsoletes(obsoletes, version)
  
        buffer = self._readFD(fd, size, filePath, progressCallback = progressCallback)
        fd.close()

        try:
            tmp_args, tmp_method = xmlrpclib.loads(buffer)
        except:
            # if there was an error decoding return as if we didnt find it
            return None
        
        # tmp_args[0] is the list of packages
        return tmp_args[0]
         

        
# need a ton of helper functions for this one, but then, it's the main one
class Up2datePackageSource(PackageSource):
    def __init__(self, server, proxyHost, cacheObject = None):
        DPRINT(server)
        self.s = server
        id = up2dateAuth.getSystemId()
        PackageSource.__init__(self, cacheObject = cacheObject)
        

    # fetch it from the network, no caching of any sort
    def getHeader(self, pkg, lite = None, msgCallback = None, progressCallback = None):
        hdr = None

        try:
            ret = self.s.up2date.header(up2dateAuth.getSystemId(), pkg)
        except KeyboardInterrupt:
            raise up2dateErrors.CommunicationError(_("Connection aborted by the user"))
        except (socket.error, socket.sslerror), e:
            if len(e.args) > 1:
                raise up2dateErrors.CommunicationError(e.args[1])
            else:
                raise up2dateErrors.CommunicationError(e.args[0])
        except cgiwrap.ProtocolError, e:
            raise up2dateErrors.CommunicationError(e.errmsg)
        except rpclib.ResponseError:
            raise up2dateErrors.CommunicationError("Broken response from the server.");
        except rpclib.Fault, f:
            raise up2dateErrors.CommunicationError(f.faultString)

        bin = ret[0]
        hdr = readHeaderBlob(bin.data)
        saveHeader(hdr)
        self.headerCache["%s-%s-%s" % (hdr['name'],
                                       hdr['version'],
                                       hdr['release'])] = hdr
        return hdr




def initHeaderSources(logininfo, serverSettings):

    httpServer = rhnHTTPlib.Server(serverSettings.httpServerUrl,
                                   proxy = serverSettings.proxyUrl,
                                   username = serverSettings.proxyUser,
                                   password = serverSettings.proxyPassword,
                                   headers = logininfo)
    #create the headerCache object all the sources will use
    localHeaderCache =  headers.HeaderCache()

    ps = PackageSourceChain(headerCacheObject = localHeaderCache)

    hcs = HeaderMemoryCache(cacheObject = localHeaderCache)
    
    headerCacheSourceInfo= {'name':'memcache', 'object': hcs}
    ps.addSourceInstance(headerCacheSourceInfo)

    hldc = LocalDisk()
    ps.addSourceInstance({'name':'localdisk', 'object':hldc})

    hds = DiskCache()
    ps.addSourceInstance({'name':'diskcache', 'object':hds})

    gds = HttpGetSource(httpServer, None, loginInfo = logininfo)
    ps.addSourceInstance({'name':'get', 'object': gds})

    return ps

def callback(total, complete):
    print "-- %s bytes of %s" % (total, complete)

# FIXME: super ugly hack that deserves to die
def updateHttpServer(packageSourceChain, logininfo, serverSettings):

    httpServer = rhnHTTPlib.Server(serverSettings.httpServerUrl,
                                   proxy = serverSettings.proxyUrl,
                                   username = serverSettings.proxyUser,
                                   password = serverSettings.proxyPassword,
                                   headers = logininfo)


    hds = HttpGetSource(httpServer, None, loginInfo = logininfo)
    packageSourceChain.addSourceInstance({'name':'get', 'object': hds})

    return packageSourceChain
                                         

def initPackageSources(logininfo, serverSettings):
    
    httpServer = rhnHTTPlib.Server(serverSettings.httpServerUrl,
                                   proxy = serverSettings.proxyUrl,
                                   username = serverSettings.proxyUser,
                                   password = serverSettings.proxyPassword,
                                   headers = logininfo)

    ps = PackageSourceChain()

    pkg_ldc = LocalDisk()
    ps.addSourceInstance({'name':'localdisk','object':pkg_ldc})

    hds = DiskCache()
    ps.addSourceInstance({'name':'diskcache', 'object':hds})

    gds = HttpGetSource(httpServer, None)
    ps.addSourceInstance({'name':'get', 'object': gds})

    return ps

def initSourcePackageSources(logininfo, serverSettings):
    
    httpServer = rhnHTTPlib.Server(serverSettings.httpServerUrl,
                                   proxy = serverSettings.proxyUrl,
                                   username = serverSettings.proxyUser,
                                   password = serverSettings.proxyPassword,
                                   headers = logininfo)

    ps = PackageSourceChain()
    
    pkg_ldc = LocalDisk()
    ps.addSourceInstance({'name':'localdisk','object':pkg_ldc})

    hds = DiskCache()
    ps.addSourceInstance({'name':'diskcache', 'object':hds})

    gds = HttpGetSource(httpServer, None)
    ps.addSourceInstance({'name':'get', 'object': gds})

    return ps

def initListPackageSources(logininfo, serverSettings):
    
    httpServer = rhnHTTPlib.Server(serverSettings.httpServerUrl,
                                   proxy = serverSettings.proxyUrl,
                                   username = serverSettings.proxyUser,
                                   password = serverSettings.proxyPassword,
                                   headers = logininfo)
    ps = PackageSourceChain()

    hds = DiskCache()
    ps.addSourceInstance({'name':'diskcache', 'object':hds})

    gds = HttpGetSource(httpServer, None)
    ps.addSourceInstance({'name':'get', 'object':gds})

    return ps



def initGetObsoletesSources(logininfo, serverSettings):

    httpServer = rhnHTTPlib.Server(serverSettings.httpServerUrl,
                                   proxy = serverSettings.proxyUrl,
                                   username = serverSettings.proxyUser,
                                   password = serverSettings.proxyPassword,
                                   headers = logininfo)
    ps = PackageSourceChain()

    hds = DiskCache()
    ps.addSourceInstance({'name':'diskcache', 'object':hds})

    gds = HttpGetSource(httpServer, None)
    ps.addSourceInstance({'name':'get', 'object':gds})

    return ps
