#!/usr/bin/python
# some high level utility stuff for rpm handling

# Client code for Update Agent
# Copyright (c) 1999-2002 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com>
#


#
#  FIXME: Some exceptions in here are currently in up2date.py
#         fix by moving to up2dateErrors.py and importing from there
#
#
        
import os
import sys
import re
        
import up2dateErrors
import up2dateLog
import up2dateUtils
import config
import rpm
import fnmatch
import gpgUtils

from translate import _, N_, cat
            
# get a reference to the log file object
log = up2dateLog.initLog()
        
#grab an instance of the config class shared
cfg = config.initUp2dateConfig()

def openrpmdb(option=0, dbpath=None):
    rpm.addMacro("_dbi_tags","Packages:Name:Basenames:Group:Requirename:Providename:Conflictname:Triggername")
    if not dbpath:
        if cfg.readEntry("dbpath"):
            dbpath = cfg.readEntry("dbpath")
        else:
            dbpath = "/var/lib/rpm/"

    rpm.addMacro("_dbpath", dbpath)

    log.log_me("Opening rpmdb in %s with option %s" % (dbpath,option))
    try:
        db = rpm.opendb(option)
    except rpm.error, e:
        raise up2dateErrors.RpmError(_("Could not open RPM database for reading.  Perhaps it is already in use?"))

    return db


# mainly here to make conflicts resolution cleaner
def findDepLocal(db, dep):
    header = None
    if dep[0] == '/':
        # Treat it as a file dependency
        try:
            hdr_arry = db.findbyfile(dep)
        except:
            hdr_arry = []
            
        for n in hdr_arry:
            header = db[n]
            break
    else:
        # Try it first as a package name
        try:
            hdr_arry = db.findbyname(dep)
        except:
            hdr_arry = []
        for n in hdr_arry:
            header = db[n]
            break
        else:
            # else try it as a soname
            try:
                hdr_arry = db.findbyprovides(dep)
            except:
                hdr_arry = []
            for n in hdr_arry:
                header = db[n]
                break
            
    if header != None:
        return header
    else:
        return None

def installedHeader(someName,db):
    h = None

    if type(someName) == type([]):
        pkgName = someName[0]
    else:
        pkgName = someName

# FIXME UGLY KLUGE FOR RPM BINDING weirdness in 7.0
    try:
	bar = db.findbyname(pkgName)
    except: 
	bar = []

    try:
        for index in bar:
            h = db[index]
            break
    except rpm.error, e:
        raise up2dateErrors.RpmError(e)

    if h != None:
        name = h['name']
        epoch = h['epoch']
        if epoch == None:
            epoch = ""
        version = h['version']
        release = h['release']
        if type(someName) == type([]):
            if (pkgName == name and
                pkgName[2] == version and
                pkgName[3] == release and
                pkgName[4] == epoch):
                return h
        else:
            if (pkgName == name):
                return h

    # not found
    return None

# check to see if a new file has a different MD5 sum
# from the package already on disk, and if true, if
# the on-disk file has been modified.
# this is a lot of args, but it helps...
def checkModified(index, fileNames, fileMD5s,installedFileNames,installedFileMD5s):
    ret = 0
    fileName = fileNames[index]

    #this is a little ugly, but the order of the filelist in the old and the new
    # pacakges arent the same.
    for j in range(len(installedFileNames)):
        if (installedFileNames[j] == fileName):
            if installedFileMD5s[j] == fileMD5s[index]:
                # the md5 of the file in the local db  is the same as the one in the
                # incoming package, so skip the rest of the check and return 0
                continue
	    # grrr, symlinks marked as config files pointing to dirs are baaad, okay?
	    if not os.path.isdir(fileName):
            	if installedFileMD5s[j] != up2dateUtils.md5sum(fileName):
                    # the local file is different than the file in the local db
                    ret = 1
            	else:
                    # not changed on disk
                    pass
            break
    return ret


def checkHeaderForFileConfigExcludes(h,package,db):
    fflag = 0

    # might as well just do this once...
    fileNames = h['filenames']
    fileMD5s = h['filemd5s']
    fileFlags = h['fileflags']

    installedHdr = installedHeader(h['name'],db)
    if not installedHdr:
        #throw a fault? should this happen?
        return None
        
    installedFileNames = installedHdr['filenames']
    installedFileMD5s = installedHdr['filemd5s']
    
    removedList = []
    if cfg.readEntry('forceInstall'):
        return None

    # bleah, have to use the index here because
    # of rpm's storing of filenames and their md5sums in parallel lists
    for f_i in range(len(fileNames)):
        # code to see if we want to disable this
        for pattern in cfg.readEntry("fileSkipList"):
            
            if fnmatch.fnmatch(fileNames[f_i],pattern):
                # got to get a better string to use here
                removedList.append((package, _("File Name/pattern")))
                fflag = 1
                break
            # if we found a matching file, no need to
            # examine the rest in this package         
            if fflag:
                break

    if cfg.readEntry("configFilesToIgnore"):
        configFilesToIgnore = cfg.readEntry("configFilesToIgnore")
    else:
        configFilesToIgnore = []

    for f_i in range(len(fileNames)):
        # Deal with config files
        if cfg.readEntry("noReplaceConfig"):
            # check for files that are config files, but skips those that
            # arent going to be replaced anyway
            # (1 << 4) == rpm.RPMFILE_NOREPLACE if it existed
            if fileFlags[f_i] & rpm.RPMFILE_CONFIG and not \
               fileFlags[f_i] & (1 << 4):
                if fileNames[f_i] not in configFilesToIgnore:
                    # check if config file and if so, if modified
                    if checkModified(f_i, fileNames, fileMD5s,
                                     installedFileNames, installedFileMD5s):
                        removedList.append((package, _("Config modified")))
                        fflag = 1
                        break

        if fflag:
            break     

    if len(removedList):
        return removedList
    else:
        return None

def checkRpmMd5(fileName):
    # FIXME
    #insert magic to "shutup" rpm here 
    # pgb 03/01/2001: not sure if any magic can fix that in RPM. :(
    
    # only newer RPMs allow this
    if "checksig" in dir(rpm):
        # RPM spews to stdout/stderr.  Redirect.
        saveStdout = os.dup(1)
        saveStderr = os.dup(2)
        redirStdout = os.open("/dev/null", os.O_WRONLY | os.O_APPEND)
        redirStderr = os.open("/dev/null", os.O_WRONLY | os.O_APPEND)
        os.dup2(redirStdout, 1)
        os.dup2(redirStderr, 2)
        # now do the rpm thing
        ret = rpm.checksig(fileName, rpm.CHECKSIG_MD5)
        # restore normal stdout and stderr
        os.dup2(saveStdout, 1)
        os.dup2(saveStderr, 2)
        # Clean up
        os.close(redirStdout)
        os.close(redirStderr)
        os.close(saveStdout)
        os.close(saveStderr)
        return ret
    else:
        # legacy 3.0.x RPM
        sigOk = 0
        lines = os.popen("/bin/rpm -Kv --nopgp --nogpg %s 2>&1" % fileName, "r").read()
        if not re.search("MD5 sum OK:", lines):
            sigOk = 1

        return sigOk



def getRpmVersion():
    db = openrpmdb()
    try:
        index = db.findbyprovides("rpm")[0]
    except:
        raise up2dateErrors.RpmError("Couldn't determine what version of rpm you are running.\nIf you get this error, try running \n\n\t\trpm --rebuilddb\n\n")
    version = ("rpm", db[index]['version'],db[index]['release'],db[index]['epoch'])
    return version

# module scope
rpm_version = getRpmVersion()

def getGPGflags():
    keyring_relocatable = 0
    if up2dateUtils.comparePackages(rpm_version, ("rpm", "4.0.4", "0", None)) >= 0:
        keyring_relocatable = 1

    if keyring_relocatable and cfg.readEntry("gpgKeyRing"):
        gpg_flags = "--homedir %s --no-default-keyring --keyring %s" % (gpgUtils.gpg_home_dir, cfg.readEntry("gpgKeyRing"))
    else:
        gpg_flags = "--homedir %s" % gpgUtils.gpg_home_dir
    return gpg_flags

# does all the magic to make sure that the signature on
# the package is one that we care aabout, etc.
def checkRpmSig(fileName):
    # must use the actual header from the real package.  The headers
    # the server returns by themselves have no signature info in them.
    fd = os.open(fileName, os.O_RDONLY)
    try:
        hdr = rpm.headerFromPackage(fd)[0]
    except:
        raise up2dateErrors.RpmError(_("Failed to read header from %s for signature verification") % fileName)
    
    os.close(fd)

    # tell rpm to use the correct keyring
    rpm.addMacro("__gpg_verify_cmd",
                 """%%{__gpg} gpg %s --batch --no-verbose --verify %%{__signature_filename} %%{__plaintext_filename}""" % (getGPGflags() ) )
    # This is for rpm 4.0.3, since I cant explicitly add a --homedir to the gpg command
    # it uses for invoking rpm, this will result in gpg getting invoked with GNUPGHOME=/root
    rpm.addMacro("_gpg_path", gpgUtils.gpg_home_dir)
    try:
        # only newer RPMs allow this
        if hdr['SIGGPG'] == None:
            return 1
        if "checksig" in dir(rpm):
            # RPM spews to stdout/stderr.  Redirect.
            saveStdout = os.dup(1)
            saveStderr = os.dup(2)
            redirStdout = os.open("/dev/null", os.O_WRONLY | os.O_APPEND)
            redirStderr = os.open("/dev/null", os.O_WRONLY | os.O_APPEND)
            os.dup2(redirStdout, 1)
            os.dup2(redirStderr, 2)
            # now do the rpm thing
            ret = rpm.checksig(fileName, rpm.CHECKSIG_GPG)
            # restore normal stdout and stderr
            os.dup2(saveStdout, 1)
            os.dup2(saveStderr, 2)
            # clean up
            os.close(redirStdout)
            os.close(redirStderr)
            os.close(saveStdout)
            os.close(saveStderr)
	    rpm.delMacro("__gpg_verify_cmd")
            rpm.delMacro("_gpg_path")
            if ret:
                return 2

        return 0
    except KeyError:
        # legacy 3.0.x RPM
        sigOk = 1
        lines = os.popen("/bin/rpm -Kv %s 2>&1" % fileName, "r").read()
        if re.search("BAD signature from \"Red Hat, Inc <security@redhat.com>\"", lines):
            sigOk = 2
        elif re.search("Good signature from \"Red Hat, Inc <security@redhat.com>\"", lines) and re.search("Fingerprint: CA20 8686 2BD6 9DFC 65F6  ECC4 2191 80CD DB42 A60E", lines):
            sigOk = 0

        return sigOk


def rpmCallback(what, amount, total, key, cb):
    if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
        pass
        #fd = os.open(key, os.O_RDONLY)
        #return fd
    elif what == rpm.RPMCALLBACK_INST_START:
        pass
    elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
        print
    elif what == rpm.RPMCALLBACK_TRANS_PROGRESS:
        if cb:
            cb(amount, total)
        else:
            print "transaction %.5s%% done\r" % ((float(amount) / total) * 100),
    elif what == rpm.RPMCALLBACK_INST_PROGRESS:
        print "installation %.5s%% done\r" % ((float(amount) / total) * 100),

    if hasattr(rpm, "RPMCALLBACK_UNPACK_ERROR"):
        if ((what == rpm.RPMCALLBACK_UNPACK_ERROR) or
                   (what == rpm.RPMCALLBACK_CPIO_ERROR)):
            pkg = "%s-%s-%s" % (key[rpm.RPMTAG_NAME],
                                key[rpm.RPMTAG_VERSION],
                                key[rpm.RPMTAG_RELEASE])

            raise up2dateErrors.RpmInstallError, "There was a fatal error installing a package", pkg


#FIXME: this looks like a good candidate for caching, since it takes a second
# or two to run, and I can call it a couple of times
def getInstalledPackageList(msgCallback = None, progressCallback = None, getArch=None):
    list = []

    if msgCallback != None:
        msgCallback(_("Getting list of packages installed on the system"))

    db = openrpmdb()

    if progressCallback != None:
        count = 1
        total = 0

        if "firstkey" in dir(db):
            key = db.firstkey()
            h = db[key]
        else:
            iterator = db.match(0)
            h = iterator.next()
        while (h):
            count = count + 1
            if "nextkey" in dir(db):
                key = db.nextkey(key)
                if key:
                    h = db[key]
                else:
                    h = None
            else:
                h = iterator.next()

        total = count
        count = 1

    if "firstkey" in dir(db):
        key = db.firstkey()
        h = db[key]
    else:
        iterator = db.match(0)
        h = iterator.next()
    while (h):
        name = h['name']
        epoch = h['epoch']
        if epoch == None:
            epoch = ""
        version = h['version']
        release = h['release']
        if getArch:
            arch = h['arch']
            list.append([name, version, release, epoch, arch])
        else:
            list.append([name, version, release, epoch])
            
        if progressCallback != None:
            progressCallback(count, total)
            count = count + 1
        if "nextkey" in dir(db):
            key = db.nextkey(key)
            if key:
                h = db[key]
            else:
                h = None
        else:
            h = iterator.next()

    list.sort()
    return list


def getInstalledObsoletes(msgCallback = None, progressCallback = None, getArch=None):
    list = []
    hash = {}

    if msgCallback != None:
        msgCallback(_("Getting list of packages installed on the system"))

    db = openrpmdb()

    if progressCallback != None:
        count = 1
        total = 0

        if "firstkey" in dir(db):
            key = db.firstkey()
            h = db[key]
        else:
            iterator = db.match(0)
            h = iterator.next()
        while (h):
            count = count + 1
            if "nextkey" in dir(db):
                key = db.nextkey(key)
                if key:
                    h = db[key]
                else:
                    h = None
            else:
                h = iterator.next()

        total = count
        count = 1

    if "firstkey" in dir(db):
        key = db.firstkey()
        h = db[key]
    else:
        iterator = db.match(0)
        h = iterator.next()
    while (h):
        obsoletes = h['obsoletes']
        name = h['name']
        version = h['version']
        release = h['release']
        nvr = "%s-%s-%s" % (name, version, release)
        if obsoletes:
            list.append((nvr, obsoletes))
            
        if progressCallback != None:
            progressCallback(count, total)
            count = count + 1
        if "nextkey" in dir(db):
            key = db.nextkey(key)
            if key:
                h = db[key]
            else:
                h = None
        else:
            h = iterator.next()

    # cat the lists together
#    obsList = []
#    for items in list:
#        obsList = obsList + items

#    obsList.sort()
    for nvr,obs in list:
        for ob in obs:
            if not hash.has_key(ob):
                hash[ob] = []
            hash[ob].append(nvr)

    return hash
#    return obsList

    
