#!/usr/bin/python -u
#
# Client code for Update Agent
# Copyright (c) 1999-2002 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com
#         Cristian Gafton <gafton@redhat.com>
#
"""Guts of the up2date client. This module provides abstration
of all the remote rpc calls that the client needs to make, includes
the dependency solve, and other assorted bits."""

import os,sys
import string
import re
import fnmatch, glob
import rpm
import socket
#import rhnxmlrpclib as xmlrpclib
import time
import StringIO
import up2dateUtils
import up2dateLog
import up2dateErrors
import up2dateAuth
import rpmSource
import rhnDefines
import rhnPackageInfo
import rhnChannel
import checkbootloader
import xmlrpclib
import rpmUtils
import rpcServer
import depSolver
import headers

# so we can use the same code with python 1.5.2 and 2.2
try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")
    

from translate import _, N_, cat

import config

cfg = config.initUp2dateConfig()

def reloadConfig():
    cfg.load()

def version():
    # substituted to the real version by the Makefile at installation time.
    return "2.8.40"


DEBUG=0
def DPRINT(string):
    if DEBUG:
        print  string

# get around cached dependencies.  Gross ugly hack.
rpm.addMacro("_dbi_tags","Packages:Name:Basenames:Group:Requirename:Providename:Conflictname:Triggername")

# this is a "gloabal" variable caching the list of all available packages
global avail_package_list
avail_package_list = []

try:
    log = up2dateLog.initLog()
except IOError, e:
    print "Unable to open log file. The error was: %s" % e
    print _("You must run the Update Agent as root.")
    sys.exit(1)



def checkForStorageDir():
    dir = cfg.readEntry("storageDir")
    if os.access(dir, os.X_OK):
        return 0
    else:
        return 1

    

headerCache = headers.HeaderCache()



def getAvailablePackageList(msgCallback=None, progressCallback=None):
    global avail_package_list


    if len(avail_package_list):
        return avail_package_list

    package_list = rhnPackageInfo.availablePackageList(msgCallback, progressCallback)

    availPkgsDict = {}
    #tmp_args,tmp_method = xmlrpclib.loads(data)
    for pkg in package_list:
        archScore = rpm.archscore(pkg[4])
        if archScore <= 0:
            # incompatible architecture
            continue

        pkgName = pkg[0]
        if not availPkgsDict.has_key(pkgName):
            availPkgsDict[pkgName] = pkg
            continue

        # Already seen this name
        ret = up2dateUtils.comparePackages(availPkgsDict[pkgName], pkg)
        if ret > 0:
            # don't care, we already have a better version
            continue
        if ret < 0:
            # Better version
            availPkgsDict[pkgName] = pkg
            continue
        # Same version, so compare the arches
        oldScore = rpm.archscore(availPkgsDict[pkgName][4])
        if oldScore <= archScore:
            continue
        # Package with a better architecture
        availPkgsDict[pkgName] = pkg

    # Sort the keys (i.e. the package names)
    avail_package_list = availPkgsDict.values()
    avail_package_list.sort()
    return avail_package_list

def getPackageSearchPath():
    dir_list = []
    dir_list.append(cfg.readEntry("storageDir"))

    dir_string = cfg.readEntry("packageDir")
    if dir_string: 
    	dir_list = dir_list + string.split(dir_string, ':')
    return dir_list


# needed for actions, maybe client to at some point
def getErrataInfo(errata_id):
    s = rpcServer.getServer()
    packages = rpcServer.doCall(s.errata.getErrataInfo, up2dateAuth.getSystemId(), errata_id)
    
    return packages


# pass the lite flag if you only want the "lite" header
def getHeader(pkg,lite=None):
    hdr = None

    hdr,type = rpcServer.doCall(rpmSource.psc.headerSource.getHeader, pkg)
    return hdr
    
    

# implicitly places headers in the headerCache.
# FIXME: i think this can die now with headers.HeaderList
def getHeaders(pkgList, msgCallback = None, progressCallback = None,
               refreshCallback = None):

    headerList = []
        
    # if we get a single header, listify it
    # double check to make sure it's a list of lists and not just
    # a single list item
    if type(pkgList) == type([]):
        if len(pkgList):
            if type(pkgList[0]) == type(""):
                tmp = [pkgList]
                pkgList = tmp
        else:
            return []
    else:
        raise TypeError, "pkgList must be a list of lists"


    # munge the list if we have the size include.
    # xml-rpc doesnt seem fond of that being included
    total = len(pkgList)

    # reset progress bar
    if progressCallback:
        progressCallback(0, total)

    if not refreshCallback:
        refreshCallback = 0

    for pkg in pkgList:
        if refreshCallback:
            refreshCallback()
        hdr = None
        hdr, sourcetype = rpcServer.doCall(rpmSource.psc.headerSource.getHeader, pkg)
        
        headerList.append(hdr)

        if progressCallback:
            progressCallback(len(headerList), total)
 
    return headerList

def removeHeader(pkg):
    # first see if it is on disk:
    fileNames = glob.glob("%s/%s.%s.hdr" %
                          (cfg.readEntry("storageDir"),
                           up2dateUtils.pkgToString(pkg),
                           up2dateUtils.archGlob()))
    for fileName in fileNames:
        try:
            os.remove(fileName)
        except:
            raise up2dateErrors.FileError(_("cannot remove %s") % fileName)

    fileNames = glob.glob("%s/%s.noarch.hdr" %
                          (cfg.readEntry("storageDir"),
                           up2dateUtils.pkgToString(pkg)))
    for fileName in fileNames:
        try:
            os.remove(fileName)
        except:
            raise up2dateErrors.FileError(_("cannot remove %s") % fileName)


def isPackageCached(pkg):

    baseFileName = "%s-%s-%s.%s.rpm" % (pkg[0], pkg[1], pkg[2], pkg[4])

    fileName = "%s/%s" % (cfg.readEntry("storageDir"),
                          baseFileName)

    if (os.access(fileName, os.R_OK)):
        size = os.stat(fileName)[6]
        if size == string.atol(pkg[5]):
            return 1
    return 0


def getPackage(pkg, msgCallback = None, progressCallback = None):

    buffer = rpcServer.doCall(rpmSource.psc.packageSource.getPackage, pkg, msgCallback, progressCallback)

    # retrieve source packages as well
    if cfg.readEntry('retrieveSource'):
        hdr = getHeader(pkg)
        srcrpm = hdr['sourcerpm']
        channel = pkg[6]
        buffer = rpcServer.doCall(rpmSource.psc.sourcePackageSource.getPackageSource,
                        channel,
                        srcrpm,
                        msgCallback,
                        progressCallback)
        
    return 0


def removePackage(pkg):
     # first see if it is on disk:
    fileNames = glob.glob("%s/%s.%s.*" %
                          (cfg.readEntry("storageDir"),
                           up2dateUtils.pkgToString(pkg),
                           up2dateUtils.archGlob()))
    for fileName in fileNames:
        log.log_me("deleting %s" % fileName)
        try:
            os.remove(fileName)
            
        except:
            raise up2dateErrors.FileError(_("cannot remove %s") % fileName)
        
    fileNames = glob.glob("%s/%s.noarch.*" %
                          (cfg.readEntry("storageDir"),
                           up2dateUtils.pkgToString(pkg)))
    for fileName in fileNames:
        log.log_me("deleting %s" % fileName)
        try:
            os.remove(fileName)
        except:
            raise up2dateErrors.FileError(_("cannot remove %s") % fileName)


def avail_package_dict():
    # returns a hash version of the Avialable Package List
    availablePkgList = getAvailablePackageList()
    pkgDict = {}
    for pkg in availablePkgList:
        pkgDict[up2dateUtils.pkgToStringArch(pkg)] = pkg
    return pkgDict


def getArchesOfAvailablePackages(availablePkgList,pkgName):
    archList = []
    for pkg in availablePkgList:
        if pkgName == pkg[0]:
            archList.append(pkg[4])

    return archList

# convience class to make removeInstalledPackagesFromList cleaner
class UpdateList:
    def __init__(self):
        self.updateDict = {}
        self.__updateIndex = None

    def nameList(self):
        return self.updateDict.keys()

    # gen a list of just the name,version,release,epoch, and arch fields
    def nvreaList(self):
        return map(lambda a: a[:5], self.updatedList())

    def nvreList(self):
        return map(lambda a: a[:4], self.updatedList())

    def appendIfNewer(self, installedPkg, pkg):
        pkgName = pkg[0]
        # do we already have a package like this?
        if not self.updateDict.has_key(pkgName):
            self.updateDict[pkgName] = pkg
            return
        # do we have a match for name-version-release
        if pkg[:4] in self.nvreList():
            # see if this is a better matching architecture
	    archScore = rpm.archscore(pkg[4])
            # incompatible arch
            if archScore <= 0:
                return
	    oldScore = rpm.archscore(self.updateDict[pkgName][4])
            if archScore < oldScore: # retain the better arch
                self.updateDict[pkgName][4] = pkg[4]
            return
        # else, we might gave an older one that needs updating
        ret = up2dateUtils.comparePackages(self.updateDict[pkgName],pkg)
        if ret > 0: # if pkg is newer
            self.updateDict[pkgName] = pkg

    def updatedList(self):
        return self.updateDict.values()

    def delete(self, pkg):
        del self.updateDict[pkg[0]]



# check if the signature matches the redhat.com one
def hasBadSignature(pkg):
    hdr = getHeader(pkg)
    fileName = "%s/%s.%s.rpm" % (cfg.readEntry("storageDir"),
                                 up2dateUtils.pkgToString(pkg),
                                 hdr['arch'])

    # all the gory bits of checking the sig is in rpmUtils now
    return rpmUtils.checkRpmSig(fileName)



# The new dryRun
def dryRun(pkgList, msgCallback = None, progressCallback = None, refreshCallback=None):
    # XXX: initialize correctly from the --verbose cmd line flag.
    debug = 0
    
    if debug:
        print "Called dryRun:", pkgList
        
    availPkgs = getAvailablePackageList()
    depsolve = depSolver.DependencySolver(pkgList, availPkgs)

    # set the callbacks wwe need
    depsolve.set_callback("msg", msgCallback)
    depsolve.set_callback("progress", progressCallback)
    depsolve.set_callback("refresh", refreshCallback)
    
    # set up the trabsaction set & all
    depsolve.setup()
    while 1:
        if debug:
            print "\nSelected so far", depsolve.selected
        ret = depsolve.solvedep()
        if debug:
            print "Got code %d from solvedep" % ret
        if ret == 0:
            break
        elif ret < 0: # not really reached
            raise up2dateErrors.DependencyError(_("Could not complete dependency resolution"))
    rc = depsolve.run()
    if progressCallback:
        progressCallback(1, 1)
    if rc and len(rc):
        msg = ""
        for code in rc:
            msg = msg + code[0] + "\n"
            raise up2dateErrors.ConflictError(_("Test install failed because of package conflicts:\n") + msg, rc)
    newList = []
    for s in depsolve.selected:
        if not s in pkgList:
            pkgList.append(s)
            newList.append(s)
    return (newList, [])
                 
def installPackages(pkgList, rpmCallback):
    try:
        log.log_me("installing packages: %s " % up2dateUtils.pprint_pkglist(pkgList))
    except:
        pass

    # a list of all the packge names to treat as kernels.
    # "hardcode" it here so people dont mess with it as much
    total = len(pkgList)

    oldInstalledPackages = rpmUtils.getInstalledPackageList()
    
    # delete any existing db, open in write mode
    db = rpmUtils.openrpmdb(option=1)
    ts = rpm.TransactionSet("/", db)

    kernels = []
    for p_i in range(total):
        hdr = getHeader(pkgList[p_i])
        if hdr:
            if "kernel" in hdr['provides']:
                ts.add(hdr,hdr, "i")
                kernels.append(hdr)
            else:
                ts.add(hdr, hdr, "u")
        else:
            raise up2dateErrors.RpmError(_("Could not find header for %s") % pkgList[p_i])

    ts.order()
    deps = ts.depcheck()
    if deps:
        raise up2dateErrors.DependencyError(_("Dependencies should have already been resolved, but they are not."), deps)

    if cfg.readEntry("justdb"):
        rc = ts.run(rpm.RPMTRANS_FLAG_JUSTDB, 0,
                    rpmCallback, cfg.readEntry("storageDir"))
    else:
        # ( 1 << 10) == rpm.RPMTRANS_FLAG_REPACKAGE
	transflag = 0
	if cfg.readEntry("enableRollbacks"):
	    transflag = (1 << 10)
        rc = ts.run(transflag , 0, rpmCallback, cfg.readEntry("storageDir"))
    if rc:
        errors = "\n"
        for e in rc:
            try:
                errors = errors + e[1] + "\n"
            except:
                errors = errors + str(e) + "\n"
        raise up2dateErrors.RpmError(_("Failed installing packages: %s") % errors)

    newInstalledPackages = rpmUtils.getInstalledPackageList()

    removedPackages = []
    addedPackages = []

    # find packages that were removed
    for pkg in oldInstalledPackages:
        if pkg not in newInstalledPackages:
            removedPackages.append(pkg)

    for pkg in newInstalledPackages:
        if pkg not in oldInstalledPackages:
            addedPackages.append(pkg)

    if len(removedPackages):
    	rhnPackageInfo.remoteDelPackages(removedPackages)
    if len(addedPackages):    
    	rhnPackageInfo.remoteAddPackages(addedPackages)

    return kernels

def removePackages(pkgList, rpmCallback):
    try:
        log.log_me("removing packages: %s " % up2dateUtils.pprint_pkglist(pkgList))
    except:
        pass  

    total = len(pkgList)

    oldInstalledPackages = rpmUtils.getInstalledPackageList()

    # delete any existing db, open in write mode
    db = rpmUtils.openrpmdb(option=1)
    ts = rpm.TransactionSet("/", db)

    # FIXME: just that a package is installed before claiming
    # it cant be removed

    remove_list = []
    for p_i in range(total):
        for pattern in cfg.readEntry("removeSkipList"):
            pkgName = "%s-%s-%s" % (pkgList[p_i][0],pkgList[p_i][1],pkgList[p_i][2])
            # check the package name against the globs in removeSkipList
            if fnmatch.fnmatch(pkgList[p_i][0], pattern):
                raise up2dateErrors.RpmRemoveSkipListError, pkgName
            else:
                remove_list.append(pkgName)

    failed_packages = {}
    for pkgName in remove_list:
        try:
            ts.remove(pkgName)
        except rpm.error, e:
            failed_packages[pkgName] = e

    # if we get failed packages, raise the hash
    # of failed packages
    if len(failed_packages.keys()):
        raise up2dateErrors.RpmRemoveError, failed_packages
    
    ts.order()
    deps = ts.depcheck()
    
    # FIXME: return failed deps if any
    if deps:
        raise up2dateErrors.DependencyError, deps

    if cfg.readEntry("justdb"):
        # do we use storage dir here?? do we need it at all?
        rc = ts.run(rpm.RPMTRANS_FLAG_JUSTDB, 0,
                    rpmCallback, cfg.readEntry("storageDir"))
    else:
        rc = ts.run(0, 0, rpmCallback, cfg.readEntry("storageDir"))
    if rc:
        errors = "\n"
        for e in rc:
            try:
                errors = errors + e[1] + "\n"
            except:
                errors = errors + str(e) + "\n"
        raise up2dateErrors.RpmError(_("Failed removing packages: %s") % errors)

    newInstalledPackages = rpmUtils.getInstalledPackageList()
    removedPackages = []
    # find packages that were removed

    for pkg in oldInstalledPackages:
        if pkg not in newInstalledPackages:
            removedPackages.append(pkg)

    if len(removedPackages):
    	rhnPackageInfo.remoteDelPackages(removedPackages)
        log.log_me("Removed packages: %s" % removedPackages)

    
    return 0

def installBootLoader(kernels):
    
    # option to disable this permanently for those using grub,etc
    if cfg.readEntry("noBootLoader"):
        return

    # dont run the new lilo if were running "justdb"
    if cfg.readEntry("justdb"):
        return

    log.log_me("Modifying bootloader config to include the new kernel info")
    kernelList = []
    for krn in kernels:
        extraInfo = krn['name']
        
        # this logics a bit weird
        if extraInfo == None:
            infoString = ""
        elif extraInfo == "kernel":
            infoString = ""
        else:
            # if its a kernel is named like "kernelsdfasd" the
            # info blurb is gonna be "kernelsdfasd". I can live
            # with that. 
            infoString = string.split(extraInfo, "-", 1)[-1:][0]
            # just in case some kook names a kernel "kernel-"
            if infoString == "":
                infoString = extraInfo
                


        verRel = "%s-%s%s" % (krn['version'], krn['release'],infoString)
        kernelList.append((verRel,extraInfo))
        log.log_me("Adding %s to bootloader config" % verRel)


    bootloader = checkbootloader.whichBootLoader()
    
    if up2dateUtils.archGlob() == 'i?86' and bootloader == "LILO":
        __install_lilo(kernelList)
    elif up2dateUtils.archGlob() == 'i?86' and bootloader == "GRUB":
        # at the moment, kernel rpms are supposed to grok grub
        # and do the Right Thing. Just a placeholder for doc purposes
        __install_grub(kernelList)
    elif up2dateUtils.archGlob() == "ia64":
        __install_elilo(kernelList)

def __install_elilo(kernelList):
    import elilocfg
        
    log.log_me("Running elilo with the new configuration")
    ret = elilocfg.installNewImages(kernelList,test=0)
    return ret

def __install_lilo(kernelList):
    import lilocfg
    
    log.log_me("Running lilo with the new configuration")
    ret = lilocfg.installNewImages(kernelList,test=0)
    return ret

def __install_grub(kernelList):
    import grubcfg

    log.log_me("Installing the kernel via grub")
    ret  = grubcfg.installNewImages(kernelList,test=0)
    return ret

