#/usr/bin/python
# Client code for Update Agent
# Copyright (c) 1999-2001 Red Hat, Inc.  Distributed under GPL.
#
# Author: Preston Brown <pbrown@redhat.com>
#         Adrian Likins <alikins@redhat.com>
#
"""utility functions for up2date"""

import re
import os
import rpm
import string
import md5
import rpmUtils
import up2dateErrors
from config import Up2dateConfig
cfg = Up2dateConfig()
cfg.load()


DEBUG=0
def DPRINT(string):
    if DEBUG:
        print  string


def rpmFlagsToOperator(flags):
    flags = flags & 0xFF
    str = ""
    if flags != 0:
        if flags & rpm.RPMSENSE_LESS:
            str = str + "<"
        if flags & rpm.RPMSENSE_GREATER:
            str = str + ">"
        if flags & rpm.RPMSENSE_EQUAL:
            str = str + "="
        if flags & rpm.RPMSENSE_SERIAL:
            str = str + "S"
    return str



def getPackageSearchPath():
    dir_list = []
    dir_list.append(cfg.readEntry("storageDir"))

    dir_string = cfg.readEntry("packageDir")
    if dir_string: 
    	dir_list = dir_list + string.split(dir_string, ':')
    return dir_list


def getProxySetting():
    proxy = None
    proxyHost = cfg.readEntry("httpProxy")
    # legacy for backwards compat
    if proxyHost == "":
        try:
            proxyHost = cfg.readEntry("pkgProxy")
        except:
            proxyHost = None
    
    if proxyHost:
        if proxyHost[:7] == "http://":
            proxy = proxyHost[7:]
        else:
            proxy = proxyHost
    
    return proxy


def pkgToString(pkg):
    return "%s-%s-%s" % (pkg[0], pkg[1], pkg[2])

def pkgToStringArch(pkg):
    return "%s-%s-%s.%s" % (pkg[0], pkg[1], pkg[2], pkg[4])

def pkglistToString(pkgs):
    packages = "("
    for pkg in pkgs:
	packages = packages + pkgToString(pkg) + ","
    packages = packages + ")"
    return packages

# compare two RPM packages
def comparePackages(pkgLabel1, pkgLabel2):
    version1 = pkgLabel1[1]
    release1 = pkgLabel1[2]
    epoch1 = pkgLabel1[3]
    version2 = pkgLabel2[1]
    release2 = pkgLabel2[2]
    epoch2 = pkgLabel2[3]

    if epoch1 == "":
        epoch1 = None
    else:
        epoch1 = "%s" % epoch1
        
    if epoch2 == "":
            epoch2 = None
    else:
        epoch2 = "%s" % epoch2
        
    return rpm.labelCompare((epoch1, version1, release1),
                            (epoch2, version2, release2))


def md5sum(fileName):
    hash = md5.new()
    
    try:
        f = open(fileName, "r")
    except:
        return ""

    fData = f.read()
    hash.update(fData)
    del fData
    f.close()
    
    hex = string.hexdigits
    md5res = ""
    for c in hash.digest():
        i = ord(c)
        md5res = md5res + hex[(i >> 4) & 0xF] + hex[i & 0xf]

    return md5res

# return a glob for your particular architecture.
def archGlob():
    if re.search("i.86", os.uname()[4]):
        return "i?86"
    elif re.search("sparc", os.uname()[4]):
        return "sparc*"
    else:
        return os.uname()[4]



def getVersion():

    if cfg.readEntry("versionOverride") != "":
        return cfg.readEntry("versionOverride")
        
    db = rpmUtils.openrpmdb()
    try:
        index = db.findbyprovides("redhat-release")[0]
    except:
        raise up2dateErrors.RpmError("Couldn't determine what version of Red Hat Linux you are running.\nIf you get this error, try running \n\n\t\trpm --rebuilddb\n\n")
    return (db[index])['version']

def pprint_pkglist(pkglist):
    if type(pkglist) == type([]):
        foo = map(lambda a : "%s-%s-%s" % (a[0],a[1],a[2]), pkglist)
    else:
        foo = "%s-%s-%s" % (pkglist[0], pkglist[1], pkglist[2])
    return foo


def freeDiskSpace():
    import statvfs

    dfInfo = os.statvfs(cfg.readEntry("storageDir"))
    return long(dfInfo[statvfs.F_BAVAIL]) * (dfInfo[statvfs.F_BSIZE])
