#!/usr/bin/python
#
# $Id: wrapperUtils.py,v 1.5.2.5 2002/10/04 07:19:01 alikins Exp $

import os   
import sys  
import string

import rpm

from translate import _, N_, cat
import up2dateErrors
import up2dateMessages
import rpmUtils
import rhnErrata
import up2dateAuth
import up2dateLog
import up2dateUtils
import rpcServer
import config


log = up2dateLog.initLog()

try:
    from rhn import rpclib
except ImportError:
    rpclib = __import__("xmlrpclib")
            
if sys.stdout.isatty():
    isatty = 1
else:                                          
    isatty = None
    
cfg = config.initUp2dateConfig()


# used for rpmCallbacks, hopefully better than having
# lots of globals lying around...
class RpmCallback:
    def __init__(self):
        self.fd = 0
        self.hashesPrinted = None
        self.lastPercent = None
        self.progressCurrent = None
        self.progressTotal = None
        self.progressCurrent = None
        self.hashesPrinted = None
        self.lastPercent = None
        self.packagesTotal = None

    def callback(self, what, amount, total, hdr, path):
        if what == rpm.RPMCALLBACK_INST_OPEN_FILE:
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            try:
                self.fd = os.open(fileName, os.O_RDONLY)
            except OSError:
                raise up2dateErrors.RpmError(_("Error opening %s") % fileName)

            return self.fd
        elif what == rpm.RPMCALLBACK_INST_CLOSE_FILE:
            os.close(self.fd)
            self.fd = 0

        elif what == rpm.RPMCALLBACK_INST_START:
            self.hashesPrinted = 0
            self.lastPercent = 0
            fileName = "%s/%s-%s-%s.%s.rpm" % (path,
                                               hdr['name'],
                                               hdr['version'],
                                               hdr['release'],
                                               hdr['arch'])
            if isatty:
                if self.progressCurrent == 0:
                        printit(_("Installing")) 
                print "%4d:%-23.23s" % (self.progressCurrent + 1, hdr['name']),
                sys.stdout.flush()
            else:
                printit(_("Installing %s") % fileName)

        elif (what == rpm.RPMCALLBACK_TRANS_PROGRESS or
              what == rpm.RPMCALLBACK_INST_PROGRESS):
            self.printRpmHash(amount, total)

        elif what == rpm.RPMCALLBACK_TRANS_START:
            self.hashesPrinted = 0
            self.lastPercent = 0
            self.progressTotal = 1
            self.progressCurrent = 0
            print "%-28s" % _("Preparing..."),
            sys.stdout.flush()

        elif what == rpm.RPMCALLBACK_TRANS_STOP:
            self.printRpmHash(1, 1)
            self.progressTotal = self.packagesTotal
            self.progressCurrent = 0

        elif (what == rpm.RPMCALLBACK_UNINST_PROGRESS or
              what == rpm.RPMCALLBACK_UNINST_START or
              what == rpm.RPMCALLBACK_UNINST_STOP):
            pass
        
        if hasattr(rpm, "RPMCALLBACK_UNPACK_ERROR"):
            if ((what == rpm.RPMCALLBACK_UNPACK_ERROR) or
                (what == rpm.RPMCALLBACK_CPIO_ERROR)):
                pkg = "%s-%s-%s" % (hdr[rpm.RPMTAG_NAME],
                                    hdr[rpm.RPMTAG_VERSION],
                                    hdr[rpm.RPMTAG_RELEASE])

                if what == rpm.RPMCALLBACK_UNPACK_ERROR:
                    raise up2dateErrors.RpmInstallError, ("There was a rpm unpack error  installing the package: %s" % pkg, pkg)
                elif what == rpm.RPMCALLBACK_CPIO_ERROR:
                    raise up2dateErrors.RpmInstallError, ("There was a cpio error installing the package: %s" % pkg, pkg)


    def printRpmHash(self,amount, total, noInc=0):
        hashesTotal = 44

        if total:
            percent = int(100 * (float(amount) / total))
        else:
            percent = 100
    
        if percent <= self.lastPercent:
            return

        self.lastPercent = percent

        if (self.hashesPrinted != hashesTotal):
            if total:
                hashesNeeded = int(hashesTotal * (float(amount) / total))
            else:
                hashesNeeded = hashesTotal

            if isatty:
                for i in range(hashesNeeded):
                    sys.stdout.write('#')

                for i in range(hashesNeeded,hashesTotal):
                    sys.stdout.write(' ')

                print "(%3d%%)" % percent, 
                for i in range(hashesTotal + 6):
                    sys.stdout.write("\b")

            self.hashesPrinted = hashesNeeded

            
            if self.hashesPrinted == hashesTotal:
                if isatty: 
                    #global progressCurrent, progressTotal
                    for i in range(1,hashesTotal):
                        sys.stdout.write("#")
                    # I dont want to increment the progress count for
                    # repackage info. Bit of a kluge
                    if not noInc:
                        self.progressCurrent = self.progressCurrent + 1
                    if self.progressTotal:
                        print " [%3d%%]" % int(100 * (float(self.progressCurrent) /
                                                      self.progressTotal))
                    else:
                        print " [%3d%%]" % 100

        sys.stdout.flush()

lastPercent = 0
def percent(amount, total, speed = 0, sec = 0):
    global lastPercent
    hashesTotal = 40

    if total:
        hashesNeeded = int(hashesTotal * (float(amount) / total))
    else:
        hashesNeeded = hashesTotal

    # dont print if were not running on a tty
    if isatty and (hashesNeeded > lastPercent or amount == total):
        for i in range(hashesNeeded):
            sys.stdout.write('#')

        sys.stdout.write('\r')

        if amount == total:
            print

    if amount == total:
        lastPercent = 0
    else:
        lastPercent = hashesNeeded


hashesPrinted = 0
progressCurrent = 0
progressTotal = 0
packagesTotal = 0
# ported from C code in RPM 2/28/01 -- PGB
def printRpmHash(amount, total):
    hashesTotal = 44
    
    if total:
        percent = int(100 * (float(amount) / total))
    else:
        percent = 100
    
    global lastPercent
    if percent <= lastPercent:
        return
    
    lastPercent = percent
    
    global hashesPrinted 
    if (hashesPrinted != hashesTotal):
        if total:
            hashesNeeded = int(hashesTotal * (float(amount) / total))
        else:
            hashesNeeded = hashesTotal
            
        if isatty:
            for i in range(hashesNeeded):
                sys.stdout.write('#')
            
            for i in range(hashesNeeded,hashesTotal):
                sys.stdout.write(' ')
                
            print "(%3d%%)" % percent, 
            for i in range(hashesTotal + 6):
                sys.stdout.write("\b")
        
        hashesPrinted = hashesNeeded
        if hashesPrinted == hashesTotal:
            if isatty: 
                global progressCurrent, progressTotal
                progressCurrent = progressCurrent + 1
                for i in range(1,hashesTotal):
                    sys.stdout.write("#")
                if progressTotal:
                    print " [%3d%%]" % int(100 * (float(progressCurrent) /
                                                  progressTotal))
                else:
                    print " [%3d%%]" % 100
    
    sys.stdout.flush()

def printRetrieveHash(amount, total, speed = 0, secs = 0):
    hashesTotal = 26
    
    if total:
        percent = int(100 * (float(amount) / total))
        hashesNeeded = int(hashesTotal * (float(amount) / total))
    else:
        percent = 100
        hashesNeeded = hashesTotal

    if isatty:
        for i in range(hashesNeeded):
            sys.stdout.write('#')

        for i in range(hashesNeeded, hashesTotal):
            sys.stdout.write(' ')

    if isatty:
        if amount == total:
            print "%-25s" % " Done."
        else:
            print "%4d k/sec, %02d:%02d:%02d rem." % \
                  (speed / 1024, secs / (60*60), (secs % 3600) / 60,
                   secs % 60),
            for i in range(hashesTotal + 25):
                sys.stdout.write("\b")
    elif amount == total:
        print "Retrieved."

def printPkg(name, shortName = None):
    if shortName:
        print "%-27.27s " % (shortName + ":"),
    else:
        print "%-27.27s " % (name + ":"),

def printit(a):
    print "\n" + a + "..."


# generic warning dialog used in several places in wrapper
def warningDialog(message, hasGui):
    if hasGui:
        try:
            from up2date_client import gui
            gui.errorWindow(message)
        except:
            print _("Unable to open gui. Try `up2date --nox`")
            print message
    else:
        print message


def printDepPackages(depPackages):
    print _("The following packages were added to your selection to satisfy dependencies:")
    print _("""
Name                                    Version        Release
--------------------------------------------------------------""")
    for pkg in depPackages:
        print "%-40s%-15s%-20s" % (pkg[0], pkg[1], pkg[2])
    print
    
def stdoutMsgCallback(msg):
    print msg

warningCallback = stdoutMsgCallback


# these functions are kind of ugly but...
def printVerboseList(availUpdates):
    print _("""
Name                                    Version        Rel     
----------------------------------------------------------""")
    for pkg in availUpdates:
        print "%-40s%-15s%-20s" % (pkg[0], pkg[1], pkg[2])
        if cfg.readEntry("verbose"):
            advisories = rhnErrata.getAdvisoryInfo(pkg)
            if advisories:
                for a in advisories:
                    indent = len(a["advisory"]) + 3
                    topic = string.strip(wrap_column(a['topic'], indent, 60))
                    print "[%s] %s\n" % (a['advisory'], topic)
            else:
                print "No advisory information available\n"
    print

def printSkippedPackages(skippedUpdates):
    print _("The following Packages were marked to be skipped by your configuration:")
    print _("""
Name                                    Version        Rel  Reason
-------------------------------------------------------------------------------""")
    for pkg,reason in skippedUpdates:
        print "%-40s%-15s%-5s%s" % (pkg[0], pkg[1], pkg[2], reason)
        if cfg.readEntry("verbose"):
            advisories = rhnErrata.getAdvisoryInfo(pkg)
            if advisories:
                for a in advisories:
                    indent = len(a["advisory"]) + 3
                    topic = string.strip(wrap_column(a['topic'], indent))
                    print "[%s] %s\n" % (a['advisory'], topic)
            else:
                print "No advisory information available\n"
    print

def printObsoletedPackages(obsoletedPackages):
    print _("The following Packages are obsoleted by newer packages:")
    print _("""
Name-Version-Release        obsoleted by      Name-Version-Release
-------------------------------------------------------------------------------""")
    for (obs,newpackages) in obsoletedPackages:
        obsstr = "%s-%s-%s" % (obs[0],obs[1],obs[2])
        newpackage = newpackages[0]
        newstr = "%s-%s-%s" % (newpackage[0], newpackage[1], newpackage[2])
        print "%-40s%-40s" % (obsstr, newstr)
        # we can have more than one package obsoleting something
        for newpackage in newpackages[1:]:
            newstr = "%s-%s-%s" % (newpackage[0], newpackage[1], newpackage[2])
            print "%-40s%-40s\n" % ("", newstr)
                                       
def  printInstalledObsoletingPackages(installedObsoletingPackages):
    print _("The following were not installed because they are obsoleted by installed packages:")
    print _("""
Name-Version-Release       obsoleted by      Name-Version-Release
-------------------------------------------------------------------------------""")
    for (obsoleted, obsoleting) in installedObsoletingPackages:
        obsstr = "%s-%s-%s" % (obsoleted[0],obsoleted[1],obsoleted[2])
        print "%-40s%-40s" % (obsstr, obsoleting[0])
        for obsoletingstr in obsoleting[1:]:
            print "%-40s%-40s" % (obsstr, obsoletingstr)
        

# wrap a long line...
def wrap_line(line, max_line_size = 100):
    if len(line) < max_line_size:
        return line
    ret = []
    l = ""
    for w in string.split(line):
        if not len(l):
            l = w
            continue
        newl = "%s %s" % (l, w)
        if len(newl) > max_line_size:
            ret.append(l)
            l = w
        else:
            l = newl
    if len(l):
        ret.append(l)
    return string.join(ret, '\n')
 
# wrap an entire piece of text
def wrap_text(txt):
    return string.join(map(wrap_line, string.split(txt, '\n')), '\n')

# wrap an entire text for diaply starting at a particular column
def wrap_column(txt, leftcol = 0, rightcol = 76):
    # get all text on a single line
    ntxt = string.join(string.split(txt), ' ')
    # wrap it in the space provided
    ntxt = wrap_line(ntxt, rightcol - leftcol)
    # add proper spaces in front of each line
    ntxt = string.join(string.split(ntxt, '\n'),
                       "\n%s" % (" " * leftcol))
    return (" " * leftcol) + ntxt

